# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/settings/about.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_About(object):
    def setupUi(self, About):
        About.setObjectName("About")
        About.resize(507, 210)
        self.about_layout = QtWidgets.QFormLayout(About)
        self.about_layout.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.about_layout.setObjectName("about_layout")
        self.version_label = QtWidgets.QLabel(About)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.version_label.setFont(font)
        self.version_label.setObjectName("version_label")
        self.about_layout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.version_label)
        self.version = QtWidgets.QLabel(About)
        self.version.setText("error")
        self.version.setObjectName("version")
        self.about_layout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.version)
        self.update_label = QtWidgets.QLabel(About)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.update_label.setFont(font)
        self.update_label.setObjectName("update_label")
        self.about_layout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.update_label)
        self.update_lbl = QtWidgets.QLabel(About)
        self.update_lbl.setText("error")
        self.update_lbl.setObjectName("update_lbl")
        self.about_layout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.update_lbl)
        self.open_browser = QtWidgets.QPushButton(About)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.open_browser.sizePolicy().hasHeightForWidth())
        self.open_browser.setSizePolicy(sizePolicy)
        self.open_browser.setObjectName("open_browser")
        self.about_layout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.open_browser)
        self.dev_label = QtWidgets.QLabel(About)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.dev_label.setFont(font)
        self.dev_label.setObjectName("dev_label")
        self.about_layout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.dev_label)
        self.dev = QtWidgets.QLabel(About)
        self.dev.setText("<a href=\'https://github.com/Dummerle\' style=\'color: #2980b9; text-decoration:none\'>Dummerle</a>\n"
"<br>\n"
"<a href=\'https://github.com/loathingkernel\' style=\'color: #2980b9; text-decoration:none\'>loathingkernel</a>")
        self.dev.setOpenExternalLinks(True)
        self.dev.setObjectName("dev")
        self.about_layout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.dev)
        self.lgd_dev_label = QtWidgets.QLabel(About)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lgd_dev_label.setFont(font)
        self.lgd_dev_label.setObjectName("lgd_dev_label")
        self.about_layout.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.lgd_dev_label)
        self.lgd_dev = QtWidgets.QLabel(About)
        self.lgd_dev.setText("<a href=\'https://github.com/derrod/\' style=\'color: #2980b9; text-decoration:none\'>derrod</a>")
        self.lgd_dev.setOpenExternalLinks(True)
        self.lgd_dev.setObjectName("lgd_dev")
        self.about_layout.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.lgd_dev)
        self.license_label = QtWidgets.QLabel(About)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.license_label.setFont(font)
        self.license_label.setObjectName("license_label")
        self.about_layout.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.license_label)
        self.license = QtWidgets.QLabel(About)
        self.license.setObjectName("license")
        self.about_layout.setWidget(6, QtWidgets.QFormLayout.FieldRole, self.license)
        self.contrib_label = QtWidgets.QLabel(About)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.contrib_label.setFont(font)
        self.contrib_label.setObjectName("contrib_label")
        self.about_layout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.contrib_label)
        self.label = QtWidgets.QLabel(About)
        self.label.setText("<a href=\'https://github.com/MultisampledNight\' style=\'color: #2980b9; text-decoration:none\'>MultisampledNight</a> <i>artwork, bug hunting, testing</i>\n"
"<br>\n"
"<a href=\'https://github.com/invertedEcho\' style=\'color: #2980b9; text-decoration:none\'>invertedEcho</a> <i>moving games, environment variables</i>\n"
"<br>\n"
"<a href=\'https://github.com/ChemicalXandco\' style=\'color: #2980b9; text-decoration:none\'>ChemicalXandco</a> <i>downloads improvements, windows integration</i>\n"
"<br>\n"
"<a href=\'https://github.com/gnanini\' style=\'color: #2980b9; text-decoration:none\'>gnanini</a> <i>steam grades</i>")
        self.label.setObjectName("label")
        self.about_layout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.label)

        self.retranslateUi(About)

    def retranslateUi(self, About):
        _translate = QtCore.QCoreApplication.translate
        About.setWindowTitle(_translate("About", "About"))
        self.version_label.setText(_translate("About", "Version"))
        self.update_label.setText(_translate("About", "Update"))
        self.open_browser.setText(_translate("About", "Download latest release"))
        self.dev_label.setText(_translate("About", "Rare Developers"))
        self.dev.setToolTip(_translate("About", "Github"))
        self.lgd_dev_label.setText(_translate("About", "Legendary Developer"))
        self.lgd_dev.setToolTip(_translate("About", "Github"))
        self.license_label.setText(_translate("About", "License"))
        self.license.setText(_translate("About", "GNU General Public License v3.0"))
        self.contrib_label.setText(_translate("About", "Rare Contributors"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    About = QtWidgets.QWidget()
    ui = Ui_About()
    ui.setupUi(About)
    About.show()
    sys.exit(app.exec_())



"""Python adaptation of https://github.com/dridk/QJsonModel

Supports Python 2 and 3 with PySide, PySide2, PyQt4 or PyQt5.
Requires https://github.com/mottosso/Qt.py

Usage:
    Use it like you would the C++ version.

    >>> import qjsonmodel
    >>> model = qjsonmodel.QJsonModel()
    >>> model.load({"key": "value"})

Test:
    Run the provided example to sanity check your Python,
    dependencies and Qt binding.

    $ python qjsonmodel.py

Changes:
    This module differs from the C++ version in the following ways.

    1. Setters and getters are replaced by Python properties
    2. Objects are sorted by default, disabled via load(sort=False)
    3. load() takes a Python dictionary as opposed to
       a string or file handle.

        - To load from a string, use built-in `json.loads()`
            >>> import json
            >>> document = json.loads("{'key': 'value'}")
            >>> model.load(document)

        - To load from a file, use `with open(fname)`
              >>> import json
              >>> with open("file.json") as f:
              ...    document = json.load(f)
              ...    model.load(document)

"""

import json

from PyQt5 import QtCore, QtWidgets


class QJsonTreeItem(object):
    def __init__(self, parent=None):
        self._parent = parent

        self._key = ""
        self._value = ""
        self._type = None
        self._children = list()

    def appendChild(self, item):
        self._children.append(item)

    def child(self, row):
        return self._children[row]

    def parent(self):
        return self._parent

    def childCount(self):
        return len(self._children)

    def row(self):
        return self._parent._children.index(self) if self._parent else 0

    @property
    def key(self):
        return self._key

    @key.setter
    def key(self, key):
        self._key = key

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        self._value = value

    @property
    def type(self):
        return self._type

    @type.setter
    def type(self, typ):
        self._type = typ

    @classmethod
    def load(self, value, parent=None, sort=True):
        rootItem = QJsonTreeItem(parent)
        rootItem.key = "root"

        if isinstance(value, dict):
            items = sorted(value.items()) if sort else value.items()

            for key, value in items:
                child = self.load(value, rootItem)
                child.key = key
                child.type = type(value)
                rootItem.appendChild(child)

        elif isinstance(value, list):
            for index, value in enumerate(value):
                child = self.load(value, rootItem)
                child.key = index
                child.type = type(value)
                rootItem.appendChild(child)

        else:
            rootItem.value = value
            rootItem.type = type(value)

        return rootItem


class QJsonModel(QtCore.QAbstractItemModel):
    def __init__(self, parent=None):
        super(QJsonModel, self).__init__(parent)

        self._rootItem = QJsonTreeItem()
        self._headers = ("key", "value")

    def clear(self):
        self.load({})

    def load(self, document):
        """Load from dictionary

        Arguments:
            document (dict): JSON-compatible dictionary

        """

        assert isinstance(
            document, (dict, list, tuple)
        ), "`document` must be of dict, list or tuple, " "not %s" % type(document)

        self.beginResetModel()

        self._rootItem = QJsonTreeItem.load(document)
        self._rootItem.type = type(document)

        self.endResetModel()

        return True

    def json(self, root=None):
        """Serialise model as JSON-compliant dictionary

        Arguments:
            root (QJsonTreeItem, optional): Serialise from here
                defaults to the the top-level item

        Returns:
            model as dict

        """

        root = root or self._rootItem
        return self.genJson(root)

    def data(self, index, role):
        if not index.isValid():
            return None

        item = index.internalPointer()

        if role == QtCore.Qt.DisplayRole:
            if index.column() == 0:
                return item.key

            if index.column() == 1:
                return item.value

        elif role == QtCore.Qt.EditRole:
            if index.column() == 1:
                return item.value

    def setData(self, index, value, role):
        if role == QtCore.Qt.EditRole:
            if index.column() == 1:
                item = index.internalPointer()
                item.value = str(value)

                self.dataChanged.emit(index, index, [QtCore.Qt.EditRole])

                return True

        return False

    def headerData(self, section, orientation, role):
        if role != QtCore.Qt.DisplayRole:
            return None

        if orientation == QtCore.Qt.Horizontal:
            return self._headers[section]

    def index(self, row, column, parent=QtCore.QModelIndex()):
        if not self.hasIndex(row, column, parent):
            return QtCore.QModelIndex()

        if not parent.isValid():
            parentItem = self._rootItem
        else:
            parentItem = parent.internalPointer()

        childItem = parentItem.child(row)
        if childItem:
            return self.createIndex(row, column, childItem)
        else:
            return QtCore.QModelIndex()

    def parent(self, index):
        if not index.isValid():
            return QtCore.QModelIndex()

        childItem = index.internalPointer()
        parentItem = childItem.parent()

        if parentItem == self._rootItem:
            return QtCore.QModelIndex()

        return self.createIndex(parentItem.row(), 0, parentItem)

    def rowCount(self, parent=QtCore.QModelIndex()):
        if parent.column() > 0:
            return 0

        if not parent.isValid():
            parentItem = self._rootItem
        else:
            parentItem = parent.internalPointer()

        return parentItem.childCount()

    def columnCount(self, parent=QtCore.QModelIndex()):
        return 2

    def flags(self, index):
        flags = super(QJsonModel, self).flags(index)

        if index.column() == 1:
            return QtCore.Qt.ItemIsEditable | flags
        else:
            return flags

    def genJson(self, item):
        nchild = item.childCount()

        if item.type is dict:
            document = {}
            for i in range(nchild):
                ch = item.child(i)
                document[ch.key] = self.genJson(ch)
            return document

        elif item.type == list:
            document = []
            for i in range(nchild):
                ch = item.child(i)
                document.append(self.genJson(ch))
            return document

        else:
            return item.value


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    view = QtWidgets.QTreeView()
    model = QJsonModel()

    view.setModel(model)

    document = json.loads(
        """\
    {
        "firstName": "John",
        "lastName": "Smith",
        "age": 25,
        "address": {
            "streetAddress": "21 2nd Street",
            "city": "New York",
            "state": "NY",
            "postalCode": "10021"
        },
        "phoneNumber": [
            {
                "type": "home",
                "number": "212 555-1234"
            },
            {
                "type": "fax",
                "number": "646 555-4567"
            }
        ]
    }
    """
    )

    model.load(document)
    model.clear()
    model.load(document)

    # Sanity check
    assert json.dumps(model.json(), sort_keys=True) == json.dumps(
        document, sort_keys=True
    )

    view.show()
    view.resize(500, 300)
    app.exec_()



import sys
from abc import abstractmethod

from PyQt5.QtCore import Qt, pyqtSlot, QCoreApplication, QSize
from PyQt5.QtGui import QCloseEvent, QKeyEvent, QKeySequence
from PyQt5.QtWidgets import (
    QDialog,
    QDialogButtonBox,
    QApplication,
    QPushButton,
    QVBoxLayout,
    QHBoxLayout,
    QWidget,
    QLayout, QSpacerItem, QSizePolicy,
)

from rare.utils.misc import icon


def dialog_title_game(text: str, app_title: str) -> str:
    return f"{text} '{app_title}'"


def dialog_title(text: str) -> str:
    return f"{text} - {QCoreApplication.instance().applicationName()}"


class BaseDialog(QDialog):

    def __init__(self, parent=None):
        super(BaseDialog, self).__init__(parent=parent)
        self.setAttribute(Qt.WA_DeleteOnClose, True)
        self.setWindowFlags(Qt.Dialog | Qt.CustomizeWindowHint | Qt.WindowTitleHint)
        self.setWindowModality(Qt.WindowModal)

    def setWindowTitle(self, a0):
        super().setWindowTitle(dialog_title(a0))

    def exec(self):
        raise RuntimeError(f"Don't use `exec()` with {type(self).__name__}")

    def exec_(self):
        raise RuntimeError(f"Don't use `exec_()` with {type(self).__name__}")

    # lk: because you will eventually find yourself back here.
    # on QDialogs the Esc key closes the dialog through keyPressEvent(),
    # which ultimately call `reject()`. Pressing the Enter/Return button
    # is a shortcut for pressing the default button and thus calling `accept()`
    # In turn  both `accept()` and `reject()` evetually call `done()`.

    # In the base dialog ignore both. In the subclasses, call the method
    # from QDialog if required, not this one.
    # `super(BaseDialog, self).keyPressEvent(a0)`
    def keyPressEvent(self, a0: QKeyEvent) -> None:
        if a0.matches(QKeySequence.Cancel):
            a0.ignore()
            return
        if a0.key() == Qt.Key_Enter or a0.key() == Qt.Key_Return:
            a0.ignore()
            return
        super().keyPressEvent(a0)

    # Using the 'X' button on the window manager comes directly here.
    # It is a spontaneous event so simply ignore it.
    def closeEvent(self, a0: QCloseEvent) -> None:
        if a0.spontaneous():
            a0.ignore()
            return
        super().closeEvent(a0)


class ButtonDialog(BaseDialog):

    def __init__(self, parent=None):
        super(ButtonDialog, self).__init__(parent=parent)

        self.reject_button = QPushButton(self)
        self.reject_button.setText(self.tr("Cancel"))
        self.reject_button.setIcon(icon("fa.remove"))
        self.reject_button.setAutoDefault(False)
        self.reject_button.clicked.connect(self.reject)

        self.accept_button = QPushButton(self)
        self.accept_button.setAutoDefault(False)
        self.accept_button.clicked.connect(self.accept)

        self.button_layout = QHBoxLayout()
        self.button_layout.addWidget(self.reject_button)
        self.button_layout.addStretch(20)
        self.button_layout.addStretch(1)
        self.button_layout.addWidget(self.accept_button)

        self.main_layout = QVBoxLayout(self)
        # lk: dirty way to set a minimum width with fixed size constraint
        spacer = QSpacerItem(
            480, self.main_layout.spacing(),
            QSizePolicy.Expanding, QSizePolicy.Fixed
        )
        self.main_layout.addItem(spacer)
        self.main_layout.addLayout(self.button_layout)
        self.main_layout.setSizeConstraint(QLayout.SetFixedSize)

    def close(self):
        raise RuntimeError(f"Don't use `close()` with {type(self).__name__}")

    def setCentralWidget(self, widget: QWidget):
        widget.layout().setContentsMargins(0, 0, 0, 0)
        self.main_layout.insertWidget(0, widget)

    def setCentralLayout(self, layout: QLayout):
        layout.setContentsMargins(0, 0, 0, 0)
        self.main_layout.insertLayout(0, layout)

    @abstractmethod
    def accept_handler(self):
        raise NotImplementedError

    @abstractmethod
    def reject_handler(self):
        raise NotImplementedError

    @abstractmethod
    def done_handler(self):
        raise NotImplementedError

    # These only apply to QDialog. If we move to QWidget for embedded dialogs
    # we have to use close() and custom handling.

    # lk: Override accept to run our abstract handling method
    def accept(self):
        self.accept_handler()
        super().accept()

    # lk: Override reject to run our abstract handling method
    def reject(self):
        self.reject_handler()
        super().reject()

    # lk: Override `done()` to to run our abstract handling method
    def done(self, a0):
        self.done_handler()
        super().done(a0)

    # lk: Ignore BaseDialog::keyPressEvent and call QDialog::keyPressEvent
    # because we handle accept and reject here.
    def keyPressEvent(self, a0: QKeyEvent) -> None:
        super(BaseDialog, self).keyPressEvent(a0)

    # lk: Ignore BaseDialog::closeEvent and call QDialog::closeEvent
    # because we handle accept and reject here.
    def closeEvent(self, a0: QCloseEvent) -> None:
        super(BaseDialog, self).closeEvent(a0)


class ActionDialog(ButtonDialog):
    def __init__(self, parent=None):
        super(ActionDialog, self).__init__(parent=parent)
        self.__reject_close = False

        self.action_button = QPushButton(self)
        self.action_button.setAutoDefault(True)
        self.action_button.clicked.connect(self.action)

        self.button_layout.insertWidget(2, self.action_button)

    def active(self) -> bool:
        return self.__reject_close

    def setActive(self, active: bool):
        self.reject_button.setDisabled(active)
        self.action_button.setDisabled(active)
        self.accept_button.setDisabled(active)
        self.__reject_close = active

    @abstractmethod
    def action_handler(self):
        raise NotImplementedError

    @pyqtSlot()
    def action(self):
        self.setActive(True)
        self.action_handler()

    # lk: Ignore all key presses if there is an ongoing action
    def keyPressEvent(self, a0: QKeyEvent) -> None:
        if self.__reject_close:
            a0.ignore()
            return
        super(BaseDialog, self).keyPressEvent(a0)

    # lk: Ignore all closeEvents if there is an ongoing action
    def closeEvent(self, a0: QCloseEvent) -> None:
        if self.__reject_close:
            a0.ignore()
            return
        super(BaseDialog, self).closeEvent(a0)


__all__ = ["dialog_title", "dialog_title_game", "BaseDialog", "ButtonDialog", "ActionDialog"]


class TestDialog(BaseDialog):
    def __init__(self, parent=None):
        super(TestDialog, self).__init__(parent=parent)

        self.accept_button = QPushButton("accept", self)
        self.reject_button = QPushButton("reject", self)
        self.action_button = QPushButton("action", self)
        self.button_box = QDialogButtonBox(Qt.Horizontal, self)
        self.button_box.addButton(self.accept_button, QDialogButtonBox.AcceptRole)
        self.button_box.addButton(self.reject_button, QDialogButtonBox.RejectRole)
        self.button_box.addButton(self.action_button, QDialogButtonBox.ActionRole)

        self.button_box.accepted.connect(self.accept)
        self.button_box.rejected.connect(self.reject)

        layout = QVBoxLayout(self)
        layout.addWidget(self.button_box)

        self.setMinimumWidth(480)

    def setWindowTitle(self, a0):
        super().setWindowTitle(dialog_title(a0))

    def close(self):
        print("in close")
        super().close()

    def closeEvent(self, a0: QCloseEvent) -> None:
        print("in closeEvent")
        if a0.spontaneous():
            print("is spontaneous")
            a0.ignore()
            return
        if self.reject_close:
            a0.ignore()
        else:
            self._on_close()
            super().closeEvent(a0)
        # super().closeEvent(a0)

    def done(self, a0):
        print(f"in done {a0}")
        return
        super().done(a0)

    def accept(self):
        print("in accept")
        self._on_accept()
        # return
        # super().accept()

    def reject(self):
        print("in reject")
        self._on_reject()
        # return
        # super().reject()

    def _on_close(self):
        print("in _on_close")

    def _on_accept(self):
        print("in _on_accepted")
        # self.close()

    def _on_reject(self):
        print("in _on_rejected")
        self.close()

    def keyPressEvent(self, a0: QKeyEvent) -> None:
        super(BaseDialog, self).keyPressEvent(a0)


def test_dialog():
    app = QApplication(sys.argv)
    dlg = TestDialog(None)
    dlg.show()
    ret = app.exec()
    sys.exit(ret)


if __name__ == "__main__":
    test_dialog()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/settings/legendary.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_LegendarySettings(object):
    def setupUi(self, LegendarySettings):
        LegendarySettings.setObjectName("LegendarySettings")
        LegendarySettings.resize(681, 456)
        LegendarySettings.setWindowTitle("LegendarySettings")
        self.legendary_layout = QtWidgets.QHBoxLayout(LegendarySettings)
        self.legendary_layout.setObjectName("legendary_layout")
        self.left_layout = QtWidgets.QVBoxLayout()
        self.left_layout.setObjectName("left_layout")
        self.install_dir_group = QtWidgets.QGroupBox(LegendarySettings)
        self.install_dir_group.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.install_dir_group.setObjectName("install_dir_group")
        self.install_dir_layout = QtWidgets.QVBoxLayout(self.install_dir_group)
        self.install_dir_layout.setObjectName("install_dir_layout")
        self.left_layout.addWidget(self.install_dir_group, 0, QtCore.Qt.AlignTop)
        self.download_group = QtWidgets.QGroupBox(LegendarySettings)
        self.download_group.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.download_group.setObjectName("download_group")
        self.download_layout = QtWidgets.QFormLayout(self.download_group)
        self.download_layout.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.download_layout.setObjectName("download_layout")
        self.max_workers_label = QtWidgets.QLabel(self.download_group)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.max_workers_label.sizePolicy().hasHeightForWidth())
        self.max_workers_label.setSizePolicy(sizePolicy)
        self.max_workers_label.setObjectName("max_workers_label")
        self.download_layout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.max_workers_label)
        self.max_workers_layout = QtWidgets.QHBoxLayout()
        self.max_workers_layout.setObjectName("max_workers_layout")
        self.max_worker_spin = QtWidgets.QSpinBox(self.download_group)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.max_worker_spin.sizePolicy().hasHeightForWidth())
        self.max_worker_spin.setSizePolicy(sizePolicy)
        self.max_worker_spin.setMinimum(0)
        self.max_worker_spin.setMaximum(16)
        self.max_worker_spin.setProperty("value", 0)
        self.max_worker_spin.setObjectName("max_worker_spin")
        self.max_workers_layout.addWidget(self.max_worker_spin)
        self.max_workers_info_label = QtWidgets.QLabel(self.download_group)
        font = QtGui.QFont()
        font.setItalic(True)
        self.max_workers_info_label.setFont(font)
        self.max_workers_info_label.setObjectName("max_workers_info_label")
        self.max_workers_layout.addWidget(self.max_workers_info_label)
        self.download_layout.setLayout(0, QtWidgets.QFormLayout.FieldRole, self.max_workers_layout)
        self.max_memory_label = QtWidgets.QLabel(self.download_group)
        self.max_memory_label.setObjectName("max_memory_label")
        self.download_layout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.max_memory_label)
        self.max_memory_layout = QtWidgets.QHBoxLayout()
        self.max_memory_layout.setObjectName("max_memory_layout")
        self.max_memory_spin = QtWidgets.QSpinBox(self.download_group)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.max_memory_spin.sizePolicy().hasHeightForWidth())
        self.max_memory_spin.setSizePolicy(sizePolicy)
        self.max_memory_spin.setMinimum(0)
        self.max_memory_spin.setMaximum(10240)
        self.max_memory_spin.setSingleStep(128)
        self.max_memory_spin.setProperty("value", 1024)
        self.max_memory_spin.setObjectName("max_memory_spin")
        self.max_memory_layout.addWidget(self.max_memory_spin)
        self.max_memory_info_label = QtWidgets.QLabel(self.download_group)
        font = QtGui.QFont()
        font.setItalic(True)
        self.max_memory_info_label.setFont(font)
        self.max_memory_info_label.setObjectName("max_memory_info_label")
        self.max_memory_layout.addWidget(self.max_memory_info_label)
        self.download_layout.setLayout(1, QtWidgets.QFormLayout.FieldRole, self.max_memory_layout)
        self.preferred_cdn_label = QtWidgets.QLabel(self.download_group)
        self.preferred_cdn_label.setObjectName("preferred_cdn_label")
        self.download_layout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.preferred_cdn_label)
        self.preferred_cdn_line = QtWidgets.QLineEdit(self.download_group)
        self.preferred_cdn_line.setObjectName("preferred_cdn_line")
        self.download_layout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.preferred_cdn_line)
        self.disable_https_label = QtWidgets.QLabel(self.download_group)
        self.disable_https_label.setObjectName("disable_https_label")
        self.download_layout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.disable_https_label)
        self.disable_https_check = QtWidgets.QCheckBox(self.download_group)
        self.disable_https_check.setText("")
        self.disable_https_check.setObjectName("disable_https_check")
        self.download_layout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.disable_https_check)
        self.left_layout.addWidget(self.download_group, 0, QtCore.Qt.AlignTop)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.left_layout.addItem(spacerItem)
        self.legendary_layout.addLayout(self.left_layout)
        self.right_layout = QtWidgets.QVBoxLayout()
        self.right_layout.setObjectName("right_layout")
        self.locale_group = QtWidgets.QGroupBox(LegendarySettings)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.locale_group.sizePolicy().hasHeightForWidth())
        self.locale_group.setSizePolicy(sizePolicy)
        self.locale_group.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.locale_group.setObjectName("locale_group")
        self.locale_layout = QtWidgets.QVBoxLayout(self.locale_group)
        self.locale_layout.setObjectName("locale_layout")
        self.right_layout.addWidget(self.locale_group, 0, QtCore.Qt.AlignTop)
        self.cleanup_group = QtWidgets.QGroupBox(LegendarySettings)
        self.cleanup_group.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.cleanup_group.setObjectName("cleanup_group")
        self.cleanup_layout = QtWidgets.QVBoxLayout(self.cleanup_group)
        self.cleanup_layout.setObjectName("cleanup_layout")
        self.clean_keep_manifests_button = QtWidgets.QPushButton(self.cleanup_group)
        self.clean_keep_manifests_button.setObjectName("clean_keep_manifests_button")
        self.cleanup_layout.addWidget(self.clean_keep_manifests_button)
        self.clean_button = QtWidgets.QPushButton(self.cleanup_group)
        self.clean_button.setObjectName("clean_button")
        self.cleanup_layout.addWidget(self.clean_button)
        self.right_layout.addWidget(self.cleanup_group)
        self.metadata_group = QtWidgets.QGroupBox(LegendarySettings)
        self.metadata_group.setObjectName("metadata_group")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.metadata_group)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.fetch_win32_check = QtWidgets.QCheckBox(self.metadata_group)
        self.fetch_win32_check.setObjectName("fetch_win32_check")
        self.verticalLayout_2.addWidget(self.fetch_win32_check)
        self.fetch_macos_check = QtWidgets.QCheckBox(self.metadata_group)
        self.fetch_macos_check.setObjectName("fetch_macos_check")
        self.verticalLayout_2.addWidget(self.fetch_macos_check)
        self.fetch_unreal_check = QtWidgets.QCheckBox(self.metadata_group)
        self.fetch_unreal_check.setObjectName("fetch_unreal_check")
        self.verticalLayout_2.addWidget(self.fetch_unreal_check)
        self.metadata_info = QtWidgets.QLabel(self.metadata_group)
        font = QtGui.QFont()
        font.setItalic(True)
        self.metadata_info.setFont(font)
        self.metadata_info.setObjectName("metadata_info")
        self.verticalLayout_2.addWidget(self.metadata_info)
        self.refresh_metadata_button = QtWidgets.QPushButton(self.metadata_group)
        self.refresh_metadata_button.setObjectName("refresh_metadata_button")
        self.verticalLayout_2.addWidget(self.refresh_metadata_button)
        self.right_layout.addWidget(self.metadata_group)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.right_layout.addItem(spacerItem1)
        self.legendary_layout.addLayout(self.right_layout)

        self.retranslateUi(LegendarySettings)

    def retranslateUi(self, LegendarySettings):
        _translate = QtCore.QCoreApplication.translate
        self.install_dir_group.setTitle(_translate("LegendarySettings", "Default installation folder"))
        self.download_group.setTitle(_translate("LegendarySettings", "Download settings"))
        self.max_workers_label.setText(_translate("LegendarySettings", "Max sorkers"))
        self.max_workers_info_label.setText(_translate("LegendarySettings", "Less is slower (0: Default)"))
        self.max_memory_label.setText(_translate("LegendarySettings", "Max shared memory"))
        self.max_memory_spin.setSuffix(_translate("LegendarySettings", "MiB"))
        self.max_memory_info_label.setText(_translate("LegendarySettings", "Less is slower (0: Default)"))
        self.preferred_cdn_label.setText(_translate("LegendarySettings", "Preferred CDN"))
        self.preferred_cdn_line.setPlaceholderText(_translate("LegendarySettings", "Default"))
        self.disable_https_label.setText(_translate("LegendarySettings", "Disable HTTPS"))
        self.locale_group.setTitle(_translate("LegendarySettings", "Locale"))
        self.cleanup_group.setTitle(_translate("LegendarySettings", "Cleanup"))
        self.clean_keep_manifests_button.setText(_translate("LegendarySettings", "Clean, but keep manifests"))
        self.clean_button.setText(_translate("LegendarySettings", "Remove everything"))
        self.metadata_group.setTitle(_translate("LegendarySettings", "Platforms"))
        self.fetch_win32_check.setText(_translate("LegendarySettings", "Include Win32 games"))
        self.fetch_macos_check.setText(_translate("LegendarySettings", "Include macOS games"))
        self.fetch_unreal_check.setText(_translate("LegendarySettings", "Include Unreal engine"))
        self.metadata_info.setText(_translate("LegendarySettings", "Restart Rare to apply"))
        self.refresh_metadata_button.setText(_translate("LegendarySettings", "Refresh metadata"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    LegendarySettings = QtWidgets.QWidget()
    ui = Ui_LegendarySettings()
    ui.setupUi(LegendarySettings)
    LegendarySettings.show()
    sys.exit(app.exec_())



import os
import platform
import subprocess
import sys
from logging import getLogger

from PyQt5.QtCore import QSettings, Qt
from PyQt5.QtWidgets import QWidget, QMessageBox

from rare.components.tabs.settings.widgets.rpc import RPCSettings
from rare.shared import LegendaryCoreSingleton
from rare.ui.components.tabs.settings.rare import Ui_RareSettings
from rare.utils.misc import (
    get_translations,
    get_color_schemes,
    set_color_pallete,
    get_style_sheets,
    set_style_sheet,
    format_size,
)
from rare.utils.paths import create_desktop_link, desktop_link_path, log_dir, desktop_links_supported

logger = getLogger("RareSettings")

languages = [("en", "English"),
             ("de", "Deutsch"),
             ("fr", "Français"),
             ("zh-Hans", "Simplified Chinese"),
             ("zh_TW", "Chinese Taiwan"),
             ("pt_BR", "Portuguese (Brazil)"),
             ("ca", "Catalan"),
             ("ru", "Russian"),
             ("tr", "Turkish"),
             ("uk", "Ukrainian")]


class RareSettings(QWidget, Ui_RareSettings):
    def __init__(self, parent=None):
        super(RareSettings, self).__init__(parent=parent)
        self.setupUi(self)
        self.core = LegendaryCoreSingleton()
        # (widget_name, option_name, default)
        self.checkboxes = [
            (self.sys_tray, "sys_tray", True),
            (self.auto_update, "auto_update", False),
            (self.confirm_start, "confirm_start", False),
            (self.auto_sync_cloud, "auto_sync_cloud", False),
            (self.notification, "notification", True),
            (self.save_size, "save_size", False),
            (self.log_games, "show_console", False),
        ]

        self.settings = QSettings()
        language = self.settings.value("language", self.core.language_code, type=str)

        # Select lang
        self.lang_select.addItems([i[1] for i in languages])
        if language in get_translations():
            index = [lang[0] for lang in languages].index(language)
            self.lang_select.setCurrentIndex(index)
        else:
            self.lang_select.setCurrentIndex(0)
        self.lang_select.currentIndexChanged.connect(self.update_lang)

        colors = get_color_schemes()
        self.color_select.addItems(colors)
        if (color := self.settings.value("color_scheme")) in colors:
            self.color_select.setCurrentIndex(self.color_select.findText(color))
            self.color_select.setDisabled(False)
            self.style_select.setDisabled(True)
        else:
            self.color_select.setCurrentIndex(0)
        self.color_select.currentIndexChanged.connect(self.on_color_select_changed)

        styles = get_style_sheets()
        self.style_select.addItems(styles)
        if (style := self.settings.value("style_sheet")) in styles:
            self.style_select.setCurrentIndex(self.style_select.findText(style))
            self.style_select.setDisabled(False)
            self.color_select.setDisabled(True)
        else:
            self.style_select.setCurrentIndex(0)
        self.style_select.currentIndexChanged.connect(self.on_style_select_changed)

        self.rpc = RPCSettings(self)
        self.right_layout.insertWidget(1, self.rpc, alignment=Qt.AlignTop)

        self.init_checkboxes(self.checkboxes)
        self.sys_tray.stateChanged.connect(
            lambda: self.settings.setValue("sys_tray", self.sys_tray.isChecked())
        )
        self.auto_update.stateChanged.connect(
            lambda: self.settings.setValue("auto_update", self.auto_update.isChecked())
        )
        self.confirm_start.stateChanged.connect(
            lambda: self.settings.setValue(
                "confirm_start", self.confirm_start.isChecked()
            )
        )
        self.auto_sync_cloud.stateChanged.connect(
            lambda: self.settings.setValue(
                "auto_sync_cloud", self.auto_sync_cloud.isChecked()
            )
        )
        self.notification.stateChanged.connect(
            lambda: self.settings.setValue("notification", self.notification.isChecked())
        )
        self.save_size.stateChanged.connect(self.save_window_size)
        self.log_games.stateChanged.connect(
            lambda: self.settings.setValue("show_console", self.log_games.isChecked())
        )

        if desktop_links_supported():
            self.desktop_link = desktop_link_path("Rare", "desktop")
            self.start_menu_link = desktop_link_path("Rare", "start_menu")
        else:
            self.desktop_link_btn.setToolTip(self.tr("Not supported"))
            self.desktop_link_btn.setDisabled(True)
            self.startmenu_link_btn.setToolTip(self.tr("Not supported"))
            self.startmenu_link_btn.setDisabled(True)
            self.desktop_link = ""
            self.start_menu_link = ""

        if self.desktop_link and self.desktop_link.exists():
            self.desktop_link_btn.setText(self.tr("Remove desktop link"))

        if self.start_menu_link and self.start_menu_link.exists():
            self.startmenu_link_btn.setText(self.tr("Remove start menu link"))

        self.desktop_link_btn.clicked.connect(self.create_desktop_link)
        self.startmenu_link_btn.clicked.connect(self.create_start_menu_link)

        self.log_dir_open_button.clicked.connect(self.open_dir)
        self.log_dir_clean_button.clicked.connect(self.clean_logdir)

        # get size of logdir
        size = sum(
            log_dir().joinpath(f).stat().st_size
            for f in log_dir().iterdir()
            if log_dir().joinpath(f).is_file()
        )
        self.log_dir_size_label.setText(format_size(size))
        # self.log_dir_clean_button.setVisible(False)
        # self.log_dir_size_label.setVisible(False)

    def clean_logdir(self):
        for f in log_dir().iterdir():
            try:
                if log_dir().joinpath(f).is_file():
                    log_dir().joinpath(f).unlink()
            except PermissionError as e:
                logger.error(e)
        size = sum(
            log_dir().joinpath(f).stat().st_size
            for f in log_dir().iterdir()
            if log_dir().joinpath(f).is_file()
        )
        self.log_dir_size_label.setText(format_size(size))

    def create_start_menu_link(self):
        try:
            if not os.path.exists(self.start_menu_link):
                if not create_desktop_link(app_name="rare_shortcut", link_type="start_menu"):
                    return
                self.startmenu_link_btn.setText(self.tr("Remove start menu link"))
            else:
                os.remove(self.start_menu_link)
                self.startmenu_link_btn.setText(self.tr("Create start menu link"))
        except PermissionError as e:
            logger.error(str(e))
            QMessageBox.warning(
                self,
                self.tr("Error"),
                self.tr("Permission error, cannot remove {}").format(self.start_menu_link),
            )

    def create_desktop_link(self):
        try:
            if not os.path.exists(self.desktop_link):
                if not create_desktop_link(app_name="rare_shortcut", link_type="desktop"):
                    return
                self.desktop_link_btn.setText(self.tr("Remove Desktop link"))
            else:
                os.remove(self.desktop_link)
                self.desktop_link_btn.setText(self.tr("Create desktop link"))
        except PermissionError as e:
            logger.error(str(e))
            logger.warning(
                self,
                self.tr("Error"),
                self.tr("Permission error, cannot remove {}").format(self.start_menu_link),
            )

    def on_color_select_changed(self, scheme):
        if scheme:
            self.style_select.setCurrentIndex(0)
            self.style_select.setDisabled(True)
            self.settings.setValue("color_scheme", self.color_select.currentText())
            set_color_pallete(self.color_select.currentText())
        else:
            self.settings.setValue("color_scheme", "")
            self.style_select.setDisabled(False)
            set_color_pallete("")

    def on_style_select_changed(self, style):
        if style:
            self.color_select.setCurrentIndex(0)
            self.color_select.setDisabled(True)
            self.settings.setValue("style_sheet", self.style_select.currentText())
            set_style_sheet(self.style_select.currentText())
        else:
            self.settings.setValue("style_sheet", "")
            self.color_select.setDisabled(False)
            set_style_sheet("")

    def open_dir(self):
        if platform.system() == "Windows":
            os.startfile(log_dir())  # pylint: disable=E1101
        else:
            opener = "open" if sys.platform == "darwin" else "xdg-open"
            subprocess.Popen([opener, log_dir()])

    def save_window_size(self):
        self.settings.setValue("save_size", self.save_size.isChecked())
        self.settings.remove("window_size")

    def update_lang(self, i: int):
        self.settings.setValue("language", languages[i][0])

    def init_checkboxes(self, checkboxes):
        for cb in checkboxes:
            widget, option, default = cb
            widget.setChecked(self.settings.value(option, default, bool))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/settings/widgets/dxvk.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DxvkSettings(object):
    def setupUi(self, DxvkSettings):
        DxvkSettings.setObjectName("DxvkSettings")
        DxvkSettings.resize(419, 185)
        self.dxvk_layout = QtWidgets.QGridLayout(DxvkSettings)
        self.dxvk_layout.setObjectName("dxvk_layout")
        self.gb_dxvk_options = QtWidgets.QGroupBox(DxvkSettings)
        self.gb_dxvk_options.setObjectName("gb_dxvk_options")
        self.layout_dxvk_options = QtWidgets.QGridLayout(self.gb_dxvk_options)
        self.layout_dxvk_options.setObjectName("layout_dxvk_options")
        self.version = QtWidgets.QCheckBox(self.gb_dxvk_options)
        self.version.setObjectName("version")
        self.layout_dxvk_options.addWidget(self.version, 0, 2, 1, 1)
        self.fps = QtWidgets.QCheckBox(self.gb_dxvk_options)
        self.fps.setObjectName("fps")
        self.layout_dxvk_options.addWidget(self.fps, 1, 0, 1, 1)
        self.memory = QtWidgets.QCheckBox(self.gb_dxvk_options)
        self.memory.setObjectName("memory")
        self.layout_dxvk_options.addWidget(self.memory, 0, 1, 1, 1)
        self.devinfo = QtWidgets.QCheckBox(self.gb_dxvk_options)
        self.devinfo.setObjectName("devinfo")
        self.layout_dxvk_options.addWidget(self.devinfo, 0, 0, 1, 1)
        self.gpuload = QtWidgets.QCheckBox(self.gb_dxvk_options)
        self.gpuload.setObjectName("gpuload")
        self.layout_dxvk_options.addWidget(self.gpuload, 1, 1, 1, 1)
        self.frametime = QtWidgets.QCheckBox(self.gb_dxvk_options)
        self.frametime.setObjectName("frametime")
        self.layout_dxvk_options.addWidget(self.frametime, 2, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(0, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.layout_dxvk_options.addItem(spacerItem, 0, 3, 3, 1)
        self.api = QtWidgets.QCheckBox(self.gb_dxvk_options)
        self.api.setObjectName("api")
        self.layout_dxvk_options.addWidget(self.api, 1, 2, 1, 1)
        self.dxvk_layout.addWidget(self.gb_dxvk_options, 2, 0, 1, 3)
        self.lbl_show_dxvk = QtWidgets.QLabel(DxvkSettings)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lbl_show_dxvk.sizePolicy().hasHeightForWidth())
        self.lbl_show_dxvk.setSizePolicy(sizePolicy)
        self.lbl_show_dxvk.setObjectName("lbl_show_dxvk")
        self.dxvk_layout.addWidget(self.lbl_show_dxvk, 0, 0, 1, 1)
        self.show_dxvk = QtWidgets.QComboBox(DxvkSettings)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.show_dxvk.sizePolicy().hasHeightForWidth())
        self.show_dxvk.setSizePolicy(sizePolicy)
        self.show_dxvk.setObjectName("show_dxvk")
        self.show_dxvk.addItem("")
        self.show_dxvk.addItem("")
        self.show_dxvk.addItem("")
        self.show_dxvk.addItem("")
        self.dxvk_layout.addWidget(self.show_dxvk, 0, 1, 1, 2)

        self.retranslateUi(DxvkSettings)

    def retranslateUi(self, DxvkSettings):
        _translate = QtCore.QCoreApplication.translate
        DxvkSettings.setWindowTitle(_translate("DxvkSettings", "DxvkSettings"))
        DxvkSettings.setTitle(_translate("DxvkSettings", "DXVK Settings"))
        self.gb_dxvk_options.setTitle(_translate("DxvkSettings", "DXVK HUD Options"))
        self.version.setText(_translate("DxvkSettings", "DXVK Version"))
        self.fps.setText(_translate("DxvkSettings", "FPS"))
        self.memory.setText(_translate("DxvkSettings", "Memory Usage"))
        self.devinfo.setText(_translate("DxvkSettings", "Device Info"))
        self.gpuload.setText(_translate("DxvkSettings", "GPU Usage"))
        self.frametime.setText(_translate("DxvkSettings", "Frame Time graph"))
        self.api.setText(_translate("DxvkSettings", "D3D Version"))
        self.lbl_show_dxvk.setText(_translate("DxvkSettings", "Show HUD"))
        self.show_dxvk.setItemText(0, _translate("DxvkSettings", "System Default"))
        self.show_dxvk.setItemText(1, _translate("DxvkSettings", "Hidden"))
        self.show_dxvk.setItemText(2, _translate("DxvkSettings", "Visible"))
        self.show_dxvk.setItemText(3, _translate("DxvkSettings", "Custom Options"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    DxvkSettings = QtWidgets.QGroupBox()
    ui = Ui_DxvkSettings()
    ui.setupUi(DxvkSettings)
    DxvkSettings.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/games/game_info/game_dlc_widget.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_GameDlcWidget(object):
    def setupUi(self, GameDlcWidget):
        GameDlcWidget.setObjectName("GameDlcWidget")
        GameDlcWidget.resize(454, 58)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(GameDlcWidget.sizePolicy().hasHeightForWidth())
        GameDlcWidget.setSizePolicy(sizePolicy)
        GameDlcWidget.setWindowTitle("GameDlcWidget")
        GameDlcWidget.setFrameShape(QtWidgets.QFrame.StyledPanel)
        GameDlcWidget.setFrameShadow(QtWidgets.QFrame.Plain)
        self.dlc_layout = QtWidgets.QHBoxLayout(GameDlcWidget)
        self.dlc_layout.setContentsMargins(0, 0, 0, 0)
        self.dlc_layout.setObjectName("dlc_layout")
        self.dlc_info = QtWidgets.QWidget(GameDlcWidget)
        self.dlc_info.setObjectName("dlc_info")
        self.dlc_info_layout = QtWidgets.QFormLayout(self.dlc_info)
        self.dlc_info_layout.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.dlc_info_layout.setContentsMargins(0, 0, 0, 0)
        self.dlc_info_layout.setObjectName("dlc_info_layout")
        self.dlc_name_label = QtWidgets.QLabel(self.dlc_info)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dlc_name_label.sizePolicy().hasHeightForWidth())
        self.dlc_name_label.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.dlc_name_label.setFont(font)
        self.dlc_name_label.setObjectName("dlc_name_label")
        self.dlc_info_layout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.dlc_name_label)
        self.dlc_name = QtWidgets.QLabel(self.dlc_info)
        self.dlc_name.setMinimumSize(QtCore.QSize(160, 0))
        self.dlc_name.setText("dlc_name")
        self.dlc_name.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.dlc_name.setObjectName("dlc_name")
        self.dlc_info_layout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.dlc_name)
        self.version_label = QtWidgets.QLabel(self.dlc_info)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.version_label.sizePolicy().hasHeightForWidth())
        self.version_label.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.version_label.setFont(font)
        self.version_label.setObjectName("version_label")
        self.dlc_info_layout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.version_label)
        self.version = QtWidgets.QLabel(self.dlc_info)
        self.version.setMinimumSize(QtCore.QSize(160, 0))
        self.version.setText("version")
        self.version.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.version.setObjectName("version")
        self.dlc_info_layout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.version)
        self.app_name_label = QtWidgets.QLabel(self.dlc_info)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.app_name_label.sizePolicy().hasHeightForWidth())
        self.app_name_label.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.app_name_label.setFont(font)
        self.app_name_label.setObjectName("app_name_label")
        self.dlc_info_layout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.app_name_label)
        self.app_name = QtWidgets.QLabel(self.dlc_info)
        self.app_name.setMinimumSize(QtCore.QSize(160, 0))
        self.app_name.setText("app_name")
        self.app_name.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.app_name.setObjectName("app_name")
        self.dlc_info_layout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.app_name)
        self.dlc_layout.addWidget(self.dlc_info)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.dlc_layout.addItem(spacerItem)
        self.action_button = QtWidgets.QPushButton(GameDlcWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.action_button.sizePolicy().hasHeightForWidth())
        self.action_button.setSizePolicy(sizePolicy)
        self.action_button.setMinimumSize(QtCore.QSize(160, 0))
        self.action_button.setText("Action")
        self.action_button.setObjectName("action_button")
        self.dlc_layout.addWidget(self.action_button, 0, QtCore.Qt.AlignBottom)
        self.dlc_layout.setStretch(1, 1)

        self.retranslateUi(GameDlcWidget)

    def retranslateUi(self, GameDlcWidget):
        _translate = QtCore.QCoreApplication.translate
        self.dlc_name_label.setText(_translate("GameDlcWidget", "DLC Name"))
        self.version_label.setText(_translate("GameDlcWidget", "Version"))
        self.app_name_label.setText(_translate("GameDlcWidget", "Application"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    GameDlcWidget = QtWidgets.QFrame()
    ui = Ui_GameDlcWidget()
    ui.setupUi(GameDlcWidget)
    GameDlcWidget.show()
    sys.exit(app.exec_())



from PyQt5.QtCore import QRect, Qt
from PyQt5.QtGui import (
    QPixmap,
    QImage,
    QPainter,
    QBrush,
    QLinearGradient,
    QPaintEvent,
    QPalette,
)

from rare.ui.components.tabs.downloads.download_widget import Ui_DownloadWidget
from rare.widgets.image_widget import ImageWidget


class DownloadWidget(ImageWidget):
    def __init__(self, parent=None):
        super(DownloadWidget, self).__init__(parent=parent)
        self.ui = Ui_DownloadWidget()
        self.ui.setupUi(self)

    """
    Painting overrides
    Let them live here until a better alternative is divised.

    This is also part of list_game_widget and maybe a
    common base can bring them together.
    """

    def prepare_pixmap(self, pixmap: QPixmap) -> QPixmap:
        device: QImage = QImage(
            pixmap.size().width() * 3,
            int(self.sizeHint().height() * pixmap.devicePixelRatioF()) + 1,
            QImage.Format_ARGB32_Premultiplied,
        )
        painter = QPainter(device)
        brush = QBrush(pixmap)
        painter.fillRect(device.rect(), brush)
        # the gradient could be cached and reused as it is expensive
        gradient = QLinearGradient(0, 0, device.width(), 0)
        gradient.setColorAt(0.15, Qt.transparent)
        gradient.setColorAt(0.5, Qt.black)
        gradient.setColorAt(0.85, Qt.transparent)
        painter.setCompositionMode(QPainter.CompositionMode_DestinationIn)
        painter.fillRect(device.rect(), gradient)
        painter.end()
        ret = QPixmap.fromImage(device)
        ret.setDevicePixelRatio(pixmap.devicePixelRatioF())
        return ret

    def setPixmap(self, pixmap: QPixmap) -> None:
        # lk: trade some possible delay and start-up time
        # lk: for faster rendering. Gradients are expensive
        # lk: so pre-generate the image
        super(DownloadWidget, self).setPixmap(self.prepare_pixmap(pixmap))

    def paint_image_empty(self, painter: QPainter, a0: QPaintEvent) -> None:
        # when pixmap object is not available yet, show a gray rectangle
        painter.setOpacity(0.5 * self._opacity)
        painter.fillRect(a0.rect(), self.palette().color(QPalette.Background))

    def paint_image_cover(self, painter: QPainter, a0: QPaintEvent) -> None:
        painter.setOpacity(self._opacity)
        color = self.palette().color(QPalette.Background).darker(75)
        painter.fillRect(self.rect(), color)
        brush = QBrush(self._pixmap)
        brush.setTransform(self._transform)
        width = int(self._pixmap.width() / self._pixmap.devicePixelRatioF())
        origin = self.width() - width
        painter.setBrushOrigin(origin, 0)
        fill_rect = QRect(origin, 0, width, self.height())
        painter.fillRect(fill_rect, brush)



from PyQt5.QtWidgets import QWidget, QVBoxLayout, QPushButton

from rare.shared import GlobalSignalsSingleton
from rare.utils.misc import ExitCodes


class DebugSettings(QWidget):
    def __init__(self, parent=None):
        super(DebugSettings, self).__init__(parent=parent)

        self.raise_runtime_exception_button = QPushButton("Raise Exception")
        self.raise_runtime_exception_button.clicked.connect(self.raise_exception)
        self.restart_button = QPushButton("Restart")
        self.restart_button.clicked.connect(
            lambda: GlobalSignalsSingleton().application.quit.emit(ExitCodes.LOGOUT)
        )

        layout = QVBoxLayout(self)
        layout.addWidget(self.raise_runtime_exception_button)
        layout.addWidget(self.restart_button)
        layout.addStretch(1)

    def raise_exception(self):
        raise RuntimeError("Debug Crash")



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/store/wishlist_widget.ui'
#
# Created by: PyQt5 UI code generator 5.15.5
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_WishlistWidget(object):
    def setupUi(self, WishlistWidget):
        WishlistWidget.setObjectName("WishlistWidget")
        WishlistWidget.resize(523, 172)
        WishlistWidget.setWindowTitle("Form")
        self.horizontalLayout = QtWidgets.QHBoxLayout(WishlistWidget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.widget = QtWidgets.QWidget(WishlistWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.widget.sizePolicy().hasHeightForWidth())
        self.widget.setSizePolicy(sizePolicy)
        self.widget.setObjectName("widget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.widget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.title_label = QtWidgets.QLabel(self.widget)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.title_label.setFont(font)
        self.title_label.setText("TextLabel")
        self.title_label.setWordWrap(True)
        self.title_label.setObjectName("title_label")
        self.verticalLayout_2.addWidget(self.title_label)
        self.developer = QtWidgets.QLabel(self.widget)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.developer.setFont(font)
        self.developer.setText("TextLabel")
        self.developer.setObjectName("developer")
        self.verticalLayout_2.addWidget(self.developer)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.discount_price = QtWidgets.QLabel(self.widget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.discount_price.sizePolicy().hasHeightForWidth())
        self.discount_price.setSizePolicy(sizePolicy)
        self.discount_price.setText("TextLabel")
        self.discount_price.setObjectName("discount_price")
        self.horizontalLayout_2.addWidget(self.discount_price)
        self.price = QtWidgets.QLabel(self.widget)
        self.price.setText("TextLabel")
        self.price.setObjectName("price")
        self.horizontalLayout_2.addWidget(self.price)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.horizontalLayout.addWidget(self.widget)
        self.delete_button = QtWidgets.QPushButton(WishlistWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.delete_button.sizePolicy().hasHeightForWidth())
        self.delete_button.setSizePolicy(sizePolicy)
        self.delete_button.setText("")
        self.delete_button.setObjectName("delete_button")
        self.horizontalLayout.addWidget(self.delete_button)

        self.retranslateUi(WishlistWidget)

    def retranslateUi(self, WishlistWidget):
        pass


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    WishlistWidget = QtWidgets.QWidget()
    ui = Ui_WishlistWidget()
    ui.setupUi(WishlistWidget)
    WishlistWidget.show()
    sys.exit(app.exec_())



from PyQt5.QtCore import QSize, pyqtSignal, pyqtSlot
from PyQt5.QtWidgets import QMenu, QTabWidget, QWidget, QWidgetAction, QShortcut, QMessageBox

from rare.shared import RareCore, LegendaryCoreSingleton, GlobalSignalsSingleton, ArgumentsSingleton
from rare.utils.misc import icon, ExitCodes
from .account import AccountWidget
from .downloads import DownloadsTab
from .games import GamesTab
from .settings import SettingsTab
from .settings.debug import DebugSettings
from .store import Shop
from .tab_widgets import MainTabBar, TabButtonWidget


class MainTabWidget(QTabWidget):
    # int: exit code
    exit_app: pyqtSignal = pyqtSignal(int)

    def __init__(self, parent):
        super(MainTabWidget, self).__init__(parent=parent)
        self.rcore = RareCore.instance()
        self.core = LegendaryCoreSingleton()
        self.signals = GlobalSignalsSingleton()
        self.args = ArgumentsSingleton()

        self.tab_bar = MainTabBar(parent=self)
        self.setTabBar(self.tab_bar)

        # Generate Tabs
        self.games_tab = GamesTab(self)
        self.games_index = self.addTab(self.games_tab, self.tr("Games"))

        # Downloads Tab after Games Tab to use populated RareCore games list
        if not self.args.offline:
            self.downloads_tab = DownloadsTab(self)
            self.downloads_index = self.addTab(self.downloads_tab, "")
            self.downloads_tab.update_title.connect(self.__on_downloads_update_title)
            self.downloads_tab.update_queues_count()
            self.setTabEnabled(self.downloads_index, not self.args.offline)

            self.store_tab = Shop(self.core)
            self.store_index = self.addTab(self.store_tab, self.tr("Store (Beta)"))
            self.setTabEnabled(self.store_index, not self.args.offline)

        # Space Tab
        space_index = self.addTab(QWidget(self), "")
        self.setTabEnabled(space_index, False)
        self.tab_bar.expanded = space_index
        # Button
        button_index = self.addTab(QWidget(self), "")
        self.setTabEnabled(button_index, False)

        self.account_widget = AccountWidget(self)
        self.account_widget.exit_app.connect(self.__on_exit_app)
        account_action = QWidgetAction(self)
        account_action.setDefaultWidget(self.account_widget)
        account_button = TabButtonWidget("mdi.account-circle", "Account", fallback_icon="fa.user")
        account_button.setMenu(QMenu())
        account_button.menu().addAction(account_action)
        self.tab_bar.setTabButton(
            button_index, MainTabBar.RightSide, account_button
        )

        self.settings_tab = SettingsTab(self)
        self.settings_index = self.addTab(self.settings_tab, icon("fa.gear"), "")
        self.settings_tab.about.update_available_ready.connect(
            lambda: self.tab_bar.setTabText(self.settings_index, "(!)")
        )

        # Open game list on click on Games tab button
        self.tabBarClicked.connect(self.mouse_clicked)
        self.setIconSize(QSize(24, 24))

        # shortcuts
        QShortcut("Alt+1", self).activated.connect(lambda: self.setCurrentIndex(self.games_index))
        if not self.args.offline:
            QShortcut("Alt+2", self).activated.connect(lambda: self.setCurrentIndex(self.downloads_index))
            QShortcut("Alt+3", self).activated.connect(lambda: self.setCurrentIndex(self.store_index))
        QShortcut("Alt+4", self).activated.connect(lambda: self.setCurrentIndex(self.settings_index))

    @pyqtSlot(int)
    def __on_downloads_update_title(self, num_downloads: int):
        self.setTabText(self.indexOf(self.downloads_tab), self.tr("Downloads ({})").format(num_downloads))

    def mouse_clicked(self, index):
        if index == self.games_index:
            self.games_tab.setCurrentWidget(self.games_tab.games_page)

    def resizeEvent(self, event):
        self.tab_bar.setMinimumWidth(self.width())
        super(MainTabWidget, self).resizeEvent(event)

    @pyqtSlot(int)
    def __on_exit_app(self, exit_code: int):
        # FIXME: Don't allow logging out if there are active downloads
        if self.downloads_tab.is_download_active:
            QMessageBox.warning(
                self,
                self.tr("Quit") if exit_code == ExitCodes.EXIT else self.tr("Logout"),
                self.tr("There are active downloads. Stop them before trying to quit."),
            )
            return
        # FIXME: End of FIXME
        if exit_code == ExitCodes.LOGOUT:
            reply = QMessageBox.question(
                self,
                self.tr("Logout"),
                self.tr("Do you really want to logout <b>{}</b>?").format(self.core.lgd.userdata.get("display_name")),
                buttons=(QMessageBox.Yes | QMessageBox.No),
                defaultButton=QMessageBox.No,
            )

            if reply == QMessageBox.Yes:
                self.core.lgd.invalidate_userdata()
            else:
                return
        self.exit_app.emit(exit_code)  # restart exit code



import shutil
from enum import Enum

from PyQt5.QtCore import QCoreApplication, pyqtSignal
from PyQt5.QtWidgets import QMessageBox

from rare.shared import LegendaryCoreSingleton
from .overlay_settings import OverlaySettings, CustomOption, ActivationStates
from rare.utils import config_helper

position_values = ["default", "top-left", "top-right", "middle-left", "middle-right", "bottom-left",
                   "bottom-right", "top-center"]


class MangoHudSettings(OverlaySettings):

    set_wrapper_activated = pyqtSignal(bool)

    def __init__(self):
        super(MangoHudSettings, self).__init__(
            [
                ("fps", QCoreApplication.translate("MangoSettings", "FPS")),
                ("frame_timing", QCoreApplication.translate("MangoSettings", "Frame Time")),
                ("cpu_stats", QCoreApplication.translate("MangoSettings", "CPU Load")),
                ("gpu_stats", QCoreApplication.translate("MangoSettings", "GPU Load")),
                ("cpu_temp", QCoreApplication.translate("MangoSettings", "CPU Temp")),
                ("gpu_temp", QCoreApplication.translate("MangoSettings", "GPU Temp")),
                ("ram", QCoreApplication.translate("MangoSettings", "Memory usage")),
                ("vram", QCoreApplication.translate("MangoSettings", "VRAM usage")),
                ("time", QCoreApplication.translate("MangoSettings", "Local Time")),
                ("version", QCoreApplication.translate("MangoSettings", "MangoHud Version")),
                ("arch", QCoreApplication.translate("MangoSettings", "System architecture")),
                ("histogram", QCoreApplication.translate("MangoSettings", "FPS Graph")),
                ("gpu_name", QCoreApplication.translate("MangoSettings", "GPU Name")),
                ("cpu_power", QCoreApplication.translate("MangoSettings", "CPU Power consumption")),
                ("gpu_power", QCoreApplication.translate("MangoSettings", "GPU Power consumption")),
            ],
            [
                (
                    CustomOption.number_input("font_size", 24, is_float=False),
                    QCoreApplication.translate("MangoSettings", "Font size")
                ),
                (
                    CustomOption.select_input("position", position_values),
                    QCoreApplication.translate("MangoSettings", "Position")
                )
            ],
            "MANGOHUD_CONFIG", "no_display", set_activation_state=self.set_activation_state
        )
        self.core = LegendaryCoreSingleton()
        self.setTitle(self.tr("MangoHud Settings"))
        self.gb_options.setTitle(self.tr("Custom options"))

    def load_settings(self, name: str):
        self.settings_updatable = False
        self.name = name
        # override
        cfg = self.core.lgd.config.get(f"{name}.env", "MANGOHUD_CONFIG", fallback="")
        activated = "mangohud" in self.core.lgd.config.get(name, "wrapper", fallback="")
        if not activated:
            self.settings_updatable = False
            self.gb_options.setDisabled(True)
            for i, checkbox in enumerate(list(self.checkboxes.values())):
                checkbox.setChecked(i < 4)
            self.show_overlay_combo.setCurrentIndex(0)
            self.settings_updatable = True
            return
        super(MangoHudSettings, self).load_settings(name)
        self.settings_updatable = False
        self.show_overlay_combo.setCurrentIndex(2)
        self.gb_options.setDisabled(False)
        for var_name, checkbox in list(self.checkboxes.items())[:4]:
            checkbox.setChecked(f"{var_name}=0" not in cfg)
        self.settings_updatable = True

    def set_activation_state(self, state: Enum):  # pylint: disable=E0202
        if state in [ActivationStates.DEFAULT, ActivationStates.HIDDEN]:
            self.set_wrapper_activated.emit(False)
            self.gb_options.setDisabled(True)

        elif state == ActivationStates.ACTIVATED:
            if not shutil.which("mangohud"):
                self.show_overlay_combo.setCurrentIndex(0)
                QMessageBox.warning(self, "Error", self.tr("Mangohud is not installed or not in path"))
                return

            cfg = self.core.lgd.config.get(f"{self.name}.env", "MANGOHUD_CONFIG", fallback="")

            split_config = cfg.split(",")
            for name in list(self.checkboxes.keys())[:4]:
                if name in split_config:
                    split_config.remove(name)
            cfg = ",".join(split_config)

            for var_name, checkbox in list(self.checkboxes.items())[:4]:  # first three are by default activated
                if not checkbox.isChecked():
                    if cfg:
                        cfg += f",{var_name}=0"
                    else:
                        cfg = f"{var_name}=0"
            if cfg:
                config_helper.add_option(f"{self.name}.env", "MANGOHUD_CONFIG", cfg)
                self.environ_changed.emit(self.config_env_var_name)
            else:
                config_helper.remove_option(f"{self.name}.env", "MANGOHUD_CONFIG")
                self.environ_changed.emit(self.config_env_var_name)

            self.set_wrapper_activated.emit(True)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/games/integrations/egl_sync_group.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_EGLSyncGroup(object):
    def setupUi(self, EGLSyncGroup):
        EGLSyncGroup.setObjectName("EGLSyncGroup")
        EGLSyncGroup.resize(424, 106)
        EGLSyncGroup.setWindowTitle("EGLSyncGroup")
        EGLSyncGroup.setCheckable(False)
        EGLSyncGroup.setChecked(False)
        self.egl_sync_layout = QtWidgets.QFormLayout(EGLSyncGroup)
        self.egl_sync_layout.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.egl_sync_layout.setObjectName("egl_sync_layout")
        self.egl_path_edit_label = QtWidgets.QLabel(EGLSyncGroup)
        self.egl_path_edit_label.setObjectName("egl_path_edit_label")
        self.egl_sync_layout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.egl_path_edit_label)
        self.egl_sync_check_label = QtWidgets.QLabel(EGLSyncGroup)
        self.egl_sync_check_label.setObjectName("egl_sync_check_label")
        self.egl_sync_layout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.egl_sync_check_label)
        self.egl_sync_check = QtWidgets.QCheckBox(EGLSyncGroup)
        font = QtGui.QFont()
        font.setItalic(True)
        self.egl_sync_check.setFont(font)
        self.egl_sync_check.setObjectName("egl_sync_check")
        self.egl_sync_layout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.egl_sync_check)
        self.import_export_layout = QtWidgets.QVBoxLayout()
        self.import_export_layout.setObjectName("import_export_layout")
        self.egl_sync_layout.setLayout(3, QtWidgets.QFormLayout.SpanningRole, self.import_export_layout)
        self.egl_path_info_label = QtWidgets.QLabel(EGLSyncGroup)
        self.egl_path_info_label.setObjectName("egl_path_info_label")
        self.egl_sync_layout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.egl_path_info_label)

        self.retranslateUi(EGLSyncGroup)

    def retranslateUi(self, EGLSyncGroup):
        _translate = QtCore.QCoreApplication.translate
        EGLSyncGroup.setTitle(_translate("EGLSyncGroup", "Sync with Epic Games Launcher"))
        self.egl_path_edit_label.setText(_translate("EGLSyncGroup", "Prefix/Manifest path"))
        self.egl_sync_check_label.setText(_translate("EGLSyncGroup", "Enable automatic sync"))
        self.egl_sync_check.setText(_translate("EGLSyncGroup", "This will immediately synchronize with EGL"))
        self.egl_path_info_label.setText(_translate("EGLSyncGroup", "Estimated path"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    EGLSyncGroup = QtWidgets.QGroupBox()
    ui = Ui_EGLSyncGroup()
    ui.setupUi(EGLSyncGroup)
    EGLSyncGroup.show()
    sys.exit(app.exec_())



from logging import getLogger
from typing import Optional

from PyQt5.QtCore import QEvent

from rare.models.game import RareGame
from rare.models.image import ImageSize
from .game_widget import GameWidget
from .icon_widget import IconWidget

logger = getLogger("IconGameWidget")


class IconGameWidget(GameWidget):
    def __init__(self, rgame: RareGame, parent=None):
        super().__init__(rgame, parent)
        self.setObjectName(f"{rgame.app_name}")
        self.setFixedSize(ImageSize.Library)
        self.ui = IconWidget()
        self.ui.setupUi(self)

        self.ui.title_label.setText(self.rgame.app_title)
        self.ui.launch_btn.clicked.connect(self._launch)
        self.ui.launch_btn.setVisible(self.rgame.is_installed)
        self.ui.install_btn.clicked.connect(self._install)
        self.ui.install_btn.setVisible(not self.rgame.is_installed)

        self.ui.launch_btn.setEnabled(self.rgame.can_launch)

        self.update_state()

        # lk: "connect" the buttons' enter/leave events to this widget
        self.installEventFilter(self)
        self.ui.launch_btn.installEventFilter(self)
        self.ui.install_btn.installEventFilter(self)

    def enterEvent(self, a0: Optional[QEvent] = None) -> None:
        if a0 is not None:
            a0.accept()
        self.ui.enterAnimation(self)

    def leaveEvent(self, a0: Optional[QEvent] = None) -> None:
        if a0 is not None:
            a0.accept()
        self.ui.leaveAnimation(self)



import platform
from typing import Union

from PyQt5.QtCore import QProcessEnvironment, pyqtSignal, QSize, Qt
from PyQt5.QtGui import QTextCursor, QFont, QCursor, QCloseEvent
from PyQt5.QtWidgets import (
    QPlainTextEdit,
    QDialog,
    QPushButton,
    QFileDialog,
    QVBoxLayout,
    QHBoxLayout,
    QSpacerItem,
    QSizePolicy, QTableWidgetItem, QHeaderView, QApplication,
)

from rare.ui.components.extra.console_env import Ui_ConsoleEnv


class Console(QDialog):
    term = pyqtSignal()
    kill = pyqtSignal()
    env: QProcessEnvironment

    def __init__(self, parent=None):
        super(Console, self).__init__(parent=parent)
        self.setAttribute(Qt.WA_DeleteOnClose, True)
        self.setWindowTitle("Rare - Console")
        self.setGeometry(0, 0, 640, 480)
        layout = QVBoxLayout()

        self.console = ConsoleEdit(self)
        layout.addWidget(self.console)

        button_layout = QHBoxLayout()

        self.env_button = QPushButton(self.tr("Show environment"))
        button_layout.addWidget(self.env_button)
        self.env_button.clicked.connect(self.show_env)

        self.save_button = QPushButton(self.tr("Save output to file"))
        button_layout.addWidget(self.save_button)
        self.save_button.clicked.connect(self.save)

        self.clear_button = QPushButton(self.tr("Clear console"))
        button_layout.addWidget(self.clear_button)
        self.clear_button.clicked.connect(self.console.clear)

        button_layout.addItem(QSpacerItem(0, 0, QSizePolicy.Expanding, QSizePolicy.Fixed))

        self.terminate_button = QPushButton(self.tr("Terminate"))
        # self.terminate_button.setVisible(platform.system() == "Windows")
        button_layout.addWidget(self.terminate_button)
        self.terminate_button.clicked.connect(lambda: self.term.emit())

        self.kill_button = QPushButton(self.tr("Kill"))
        # self.kill_button.setVisible(platform.system() == "Windows")
        button_layout.addWidget(self.kill_button)
        self.kill_button.clicked.connect(lambda: self.kill.emit())

        layout.addLayout(button_layout)

        self.setLayout(layout)

        self.env_variables = ConsoleEnv(self)
        self.env_variables.hide()

        self.accept_close = False

    def show(self) -> None:
        super(Console, self).show()
        self.center_window()

    def center_window(self):
        # get the margins of the decorated window
        margins = self.windowHandle().frameMargins()
        # get the screen the cursor is on
        current_screen = QApplication.screenAt(QCursor.pos())
        if not current_screen:
            current_screen = QApplication.primaryScreen()
        # get the available screen geometry (excludes panels/docks)
        screen_rect = current_screen.availableGeometry()
        decor_width = margins.left() + margins.right()
        decor_height = margins.top() + margins.bottom()
        window_size = QSize(self.width(), self.height()).boundedTo(
            screen_rect.size() - QSize(decor_width, decor_height)
        )

        self.resize(window_size)
        self.move(
            screen_rect.center()
            - self.rect().adjusted(0, 0, decor_width, decor_height).center()
        )

    def save(self):
        file, ok = QFileDialog.getSaveFileName(
            self, "Save output", "", "Log Files (*.log);;All Files (*)"
        )
        if ok:
            if "." not in file:
                file += ".log"
            with open(file, "w") as f:
                f.write(self.console.toPlainText())
                f.close()
                self.save_button.setText(self.tr("Saved"))

    def set_env(self, env: QProcessEnvironment):
        self.env = env

    def show_env(self):
        self.env_variables.setTable(self.env)
        self.env_variables.show()

    def log(self, text: str, end: str = "\n"):
        self.console.log(text + end)

    def error(self, text, end: str = "\n"):
        self.console.error(text + end)

    def on_process_exit(self, app_title: str, status: Union[int, str]):
        self.error(
            self.tr("Application \"{}\" finished with \"{}\"\n").format(app_title, status)
        )
        self.accept_close = True

    def reject(self) -> None:
        self.close()

    def accept(self) -> None:
        self.close()

    def closeEvent(self, a0: QCloseEvent) -> None:
        if self.accept_close:
            super(Console, self).closeEvent(a0)
            a0.accept()
        else:
            self.showMinimized()
            a0.ignore()


class ConsoleEnv(QDialog):

    def __init__(self, parent=None):
        super(ConsoleEnv, self).__init__(parent=parent)
        self.setAttribute(Qt.WA_DeleteOnClose, False)
        self.ui = Ui_ConsoleEnv()
        self.ui.setupUi(self)

    def setTable(self, env: QProcessEnvironment):
        self.ui.table.clearContents()
        self.ui.table.setRowCount(len(env.keys()))

        for idx, key in enumerate(env.keys()):
            self.ui.table.setItem(idx, 0, QTableWidgetItem(env.keys()[idx]))
            self.ui.table.setItem(idx, 1, QTableWidgetItem(env.value(env.keys()[idx])))

        self.ui.table.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeToContents)


class ConsoleEdit(QPlainTextEdit):

    def __init__(self, parent=None):
        super(ConsoleEdit, self).__init__(parent=parent)
        self.setReadOnly(True)
        font = QFont("Monospace")
        font.setStyleHint(QFont.Monospace)
        self.setFont(font)
        self._cursor_output = self.textCursor()

    def log(self, text):
        html = f"<p style=\"color:#BBB;white-space:pre\">{text}</p>"
        self._cursor_output.insertHtml(html)
        self.scroll_to_last_line()

    def error(self, text):
        html = f"<p style=\"color:#eee;white-space:pre\">{text}</p>"
        self._cursor_output.insertHtml(html)
        self.scroll_to_last_line()

    def scroll_to_last_line(self):
        cursor = self.textCursor()
        cursor.movePosition(QTextCursor.End)
        cursor.movePosition(
            QTextCursor.Up if cursor.atBlockStart() else QTextCursor.StartOfLine
        )
        self.setTextCursor(cursor)



from logging import getLogger
from typing import Optional

from PyQt5.QtCore import pyqtSignal, Qt, QThreadPool, pyqtSlot
from PyQt5.QtWidgets import QWidget, QFrame
from legendary.models.downloading import AnalysisResult
from legendary.models.game import Game, InstalledGame
from qtawesome import icon

from rare.models.install import InstallQueueItemModel, InstallOptionsModel, InstallDownloadModel
from rare.shared import RareCore, ImageManagerSingleton
from rare.shared.workers.install_info import InstallInfoWorker
from rare.ui.components.tabs.downloads.queue_base_widget import Ui_QueueBaseWidget
from rare.ui.components.tabs.downloads.queue_info_widget import Ui_QueueInfoWidget
from rare.utils.misc import format_size, widget_object_name, elide_text
from rare.widgets.image_widget import ImageWidget, ImageSize

logger = getLogger("DownloadWidgets")


class QueueInfoWidget(QWidget):
    def __init__(
        self,
        game: Optional[Game],
        igame: Optional[InstalledGame],
        analysis: Optional[AnalysisResult] = None,
        old_igame: Optional[InstalledGame] = None,
        parent=None,
    ):
        super(QueueInfoWidget, self).__init__(parent=parent)
        self.ui = Ui_QueueInfoWidget()
        self.ui.setupUi(self)

        self.image_manager = ImageManagerSingleton()

        self.image = ImageWidget(self)
        self.image.setFixedSize(ImageSize.Icon)
        self.ui.image_layout.addWidget(self.image)

        self.ui.queue_info_layout.setAlignment(Qt.AlignTop)

        if game and igame:
            self.update_information(game, igame, analysis, old_igame)
        else:
            self.ui.title.setText("...")
            self.ui.remote_version.setText("...")
            self.ui.local_version.setText("...")
            self.ui.dl_size.setText("...")
            self.ui.install_size.setText("...")

        if old_igame:
            self.ui.title.setText(old_igame.title)
            self.image.setPixmap(self.image_manager.get_pixmap(old_igame.app_name, color=True))

    def update_information(self, game, igame, analysis, old_igame):
        self.ui.title.setText(game.app_title)
        self.ui.remote_version.setText(
            elide_text(self.ui.remote_version, old_igame.version if old_igame else game.app_version(igame.platform))
        )
        self.ui.local_version.setText(elide_text(self.ui.local_version, igame.version))
        self.ui.dl_size.setText(format_size(analysis.dl_size) if analysis else "")
        self.ui.install_size.setText(format_size(analysis.install_size) if analysis else "")
        self.image.setPixmap(self.image_manager.get_pixmap(game.app_name, color=True))


class UpdateWidget(QFrame):
    enqueue = pyqtSignal(InstallOptionsModel)

    def __init__(self, game: Game, igame: InstalledGame, parent=None):
        super(UpdateWidget, self).__init__(parent=parent)
        self.ui = Ui_QueueBaseWidget()
        self.ui.setupUi(self)
        # lk: setObjectName has to be after `setupUi` because it is also set in that function
        self.setObjectName(widget_object_name(self, game.app_name))

        self.game = game
        self.igame = igame

        self.ui.queue_buttons.setVisible(False)
        self.ui.move_buttons.setVisible(False)

        self.info_widget = QueueInfoWidget(game, igame, parent=self)
        self.ui.info_layout.addWidget(self.info_widget)

        self.ui.update_button.clicked.connect(lambda: self.update_game(True))
        self.ui.settings_button.clicked.connect(lambda: self.update_game(False))

    def update_game(self, auto: bool):
        self.ui.update_button.setDisabled(True)
        self.ui.settings_button.setDisabled(True)
        self.enqueue.emit(InstallOptionsModel(app_name=self.game.app_name, update=True, silent=auto))  # True if settings

    def set_enabled(self, enabled: bool):
        self.ui.update_button.setEnabled(enabled)
        self.ui.settings_button.setEnabled(enabled)

    def version(self) -> str:
        return self.game.app_version(self.igame.platform)


class QueueWidget(QFrame):
    # str: app_name
    move_up = pyqtSignal(str)
    # str: app_name
    move_down = pyqtSignal(str)
    # str: app_name
    remove = pyqtSignal(str)
    # InstallQueueItemModel
    force = pyqtSignal(InstallQueueItemModel)

    def __init__(self, item: InstallQueueItemModel, old_igame: InstalledGame, parent=None):
        super(QueueWidget, self).__init__(parent=parent)
        self.ui = Ui_QueueBaseWidget()
        self.ui.setupUi(self)
        # lk: setObjectName has to be after `setupUi` because it is also set in that function
        self.setObjectName(widget_object_name(self, item.options.app_name))

        if not item:
            self.ui.queue_buttons.setEnabled(False)
            worker = InstallInfoWorker(RareCore.instance().core(), item.options)
            worker.signals.result.connect(self.__update_info)
            worker.signals.failed.connect(
                lambda m: logger.error(f"Failed to requeue download for {item.options.app_name} with error: {m}")
            )
            worker.signals.failed.connect(lambda m: self.remove.emit(item.options.app_name))
            worker.signals.finished.connect(
                lambda: logger.info(f"Download requeue worker finished for {item.options.app_name}")
            )
            QThreadPool.globalInstance().start(worker)
            self.info_widget = QueueInfoWidget(None, None, None, old_igame, parent=self)
        else:
            self.info_widget = QueueInfoWidget(
                item.download.game, item.download.igame, item.download.analysis, old_igame, parent=self
            )
        self.ui.info_layout.addWidget(self.info_widget)
        self.ui.update_buttons.setVisible(False)

        self.old_igame = old_igame
        self.item = item

        self.ui.move_up_button.setIcon(icon("fa.arrow-up"))
        self.ui.move_up_button.clicked.connect(
            lambda: self.move_up.emit(self.item.options.app_name)
        )

        self.ui.move_down_button.setIcon(icon("fa.arrow-down"))
        self.ui.move_down_button.clicked.connect(
            lambda: self.move_down.emit(self.item.options.app_name)
        )

        self.ui.remove_button.clicked.connect(lambda: self.remove.emit(self.item.options.app_name))
        self.ui.force_button.clicked.connect(lambda: self.force.emit(self.item))

    @pyqtSlot(InstallDownloadModel)
    def __update_info(self, download: InstallDownloadModel):
        self.item.download = download
        if self.item:
            self.info_widget.update_information(download.game, download.igame, download.analysis, self.old_igame)
        self.ui.queue_buttons.setEnabled(bool(self.item))

    def toggle_arrows(self, index: int, length: int):
        self.ui.move_up_button.setEnabled(bool(index))
        self.ui.move_down_button.setEnabled(bool(length - (index + 1)))



import os
import shutil
from typing import Tuple

from PyQt5.QtWidgets import QHBoxLayout, QCheckBox, QFileDialog

from rare.shared import LegendaryCoreSingleton
from rare.utils import config_helper
from rare.widgets.indicator_edit import PathEdit, IndicatorReasonsCommon


class PreLaunchSettings(QHBoxLayout):
    app_name: str

    def __init__(self):
        super(PreLaunchSettings, self).__init__()
        self.core = LegendaryCoreSingleton()
        self.edit = PathEdit(
            path="",
            placeholder=self.tr("Path to script"),
            file_mode=QFileDialog.ExistingFile,
            edit_func=self.edit_command,
            save_func=self.save_pre_launch_command,
        )
        self.layout().addWidget(self.edit)

        self.wait_check = QCheckBox(self.tr("Wait for finish"))
        self.layout().addWidget(self.wait_check)
        self.wait_check.stateChanged.connect(self.save_wait_finish)

    def edit_command(self, text: str) -> Tuple[bool, str, int]:
        if not text.strip():
            return True, text, IndicatorReasonsCommon.VALID

        if not os.path.isfile(text.split()[0]) and not shutil.which(text.split()[0]):
            return False, text, IndicatorReasonsCommon.FILE_NOT_EXISTS
        else:
            return True, text, IndicatorReasonsCommon.VALID

    def save_pre_launch_command(self, text):
        if text:
            config_helper.add_option(self.app_name, "pre_launch_command", text)
            self.wait_check.setDisabled(False)
        else:
            config_helper.remove_option(self.app_name, "pre_launch_command")
            self.wait_check.setDisabled(True)
            config_helper.remove_option(self.app_name, "pre_launch_wait")

    def save_wait_finish(self):
        config_helper.add_option(self.app_name, "pre_launch_wait", str(self.wait_check.isChecked()).lower())

    def load_settings(self, app_name):
        self.app_name = app_name

        command = self.core.lgd.config.get(app_name, "pre_launch_command", fallback="")
        self.edit.setText(command)

        wait = self.core.lgd.config.getboolean(app_name, "pre_launch_wait", fallback=False)
        self.wait_check.setChecked(wait)

        self.wait_check.setEnabled(bool(command))



from logging import getLogger

from PyQt5.QtCore import QSettings, Qt, pyqtSlot
from PyQt5.QtGui import QShowEvent
from PyQt5.QtWidgets import QStackedWidget, QVBoxLayout, QWidget, QScrollArea, QFrame

from rare.models.game import RareGame
from rare.shared import (
    LegendaryCoreSingleton,
    GlobalSignalsSingleton,
    ArgumentsSingleton,
    ImageManagerSingleton,
)
from rare.shared import RareCore
from rare.widgets.library_layout import LibraryLayout
from rare.widgets.sliding_stack import SlidingStackedWidget
from .game_info import GameInfoTabs
from .game_widgets import LibraryWidgetController
from .game_widgets.icon_game_widget import IconGameWidget
from .game_widgets.list_game_widget import ListGameWidget
from .head_bar import GameListHeadBar
from .integrations import IntegrationsTabs

logger = getLogger("GamesTab")


class GamesTab(QStackedWidget):
    def __init__(self, parent=None):
        super(GamesTab, self).__init__(parent=parent)
        self.rcore = RareCore.instance()
        self.core = LegendaryCoreSingleton()
        self.signals = GlobalSignalsSingleton()
        self.args = ArgumentsSingleton()
        self.image_manager = ImageManagerSingleton()
        self.settings = QSettings()

        self.games_page = QWidget(parent=self)
        games_page_layout = QVBoxLayout(self.games_page)
        self.addWidget(self.games_page)

        self.head_bar = GameListHeadBar(parent=self.games_page)
        self.head_bar.goto_import.connect(self.show_import)
        self.head_bar.goto_egl_sync.connect(self.show_egl_sync)
        self.head_bar.goto_eos_ubisoft.connect(self.show_eos_ubisoft)
        games_page_layout.addWidget(self.head_bar)

        self.game_info_page = GameInfoTabs(self)
        self.game_info_page.back_clicked.connect(lambda: self.setCurrentWidget(self.games_page))
        self.game_info_page.import_clicked.connect(self.show_import)
        self.addWidget(self.game_info_page)

        self.integrations_page = IntegrationsTabs(self)
        self.integrations_page.back_clicked.connect(lambda: self.setCurrentWidget(self.games_page))
        self.addWidget(self.integrations_page)

        self.view_stack = SlidingStackedWidget(self.games_page)
        self.view_stack.setFrameStyle(QFrame.NoFrame)

        self.icon_view_scroll = QScrollArea(self.view_stack)
        self.icon_view_scroll.setWidgetResizable(True)
        self.icon_view_scroll.setFrameShape(QFrame.StyledPanel)
        self.icon_view_scroll.horizontalScrollBar().setDisabled(True)

        self.list_view_scroll = QScrollArea(self.view_stack)
        self.list_view_scroll.setWidgetResizable(True)
        self.list_view_scroll.setFrameShape(QFrame.StyledPanel)
        self.list_view_scroll.horizontalScrollBar().setDisabled(True)

        self.icon_view = QWidget(self.icon_view_scroll)
        icon_view_layout = LibraryLayout(self.icon_view)
        icon_view_layout.setSpacing(9)
        icon_view_layout.setContentsMargins(0, 13, 0, 13)
        icon_view_layout.setAlignment(Qt.AlignTop)

        self.list_view = QWidget(self.list_view_scroll)
        list_view_layout = QVBoxLayout(self.list_view)
        list_view_layout.setContentsMargins(3, 3, 9, 3)
        list_view_layout.setAlignment(Qt.AlignTop)

        self.library_controller = LibraryWidgetController(self.icon_view, self.list_view, self)
        self.icon_view_scroll.setWidget(self.icon_view)
        self.list_view_scroll.setWidget(self.list_view)
        self.view_stack.addWidget(self.icon_view_scroll)
        self.view_stack.addWidget(self.list_view_scroll)
        games_page_layout.addWidget(self.view_stack)

        if not self.settings.value("icon_view", True, bool):
            self.view_stack.setCurrentWidget(self.list_view_scroll)
            self.head_bar.view.list()
        else:
            self.view_stack.setCurrentWidget(self.icon_view_scroll)

        self.head_bar.search_bar.textChanged.connect(lambda x: self.filter_games("", x))
        self.head_bar.search_bar.textChanged.connect(self.scroll_to_top)
        self.head_bar.filterChanged.connect(self.filter_games)
        self.head_bar.filterChanged.connect(self.scroll_to_top)
        self.head_bar.refresh_list.clicked.connect(self.library_controller.update_list)
        self.head_bar.view.toggled.connect(self.toggle_view)

        self.active_filter: str = self.head_bar.filter.currentData(Qt.UserRole)

        # signals
        self.signals.game.installed.connect(self.update_count_games_label)
        self.signals.game.uninstalled.connect(self.update_count_games_label)

        self.init = True

    def showEvent(self, a0: QShowEvent):
        if a0.spontaneous() or not self.init:
            return super().showEvent(a0)
        self.setup_game_list()
        self.init = False
        return super().showEvent(a0)

    @pyqtSlot()
    def scroll_to_top(self):
        self.icon_view_scroll.verticalScrollBar().setSliderPosition(
            self.icon_view_scroll.verticalScrollBar().minimum()
        )
        self.list_view_scroll.verticalScrollBar().setSliderPosition(
            self.list_view_scroll.verticalScrollBar().minimum()
        )

    @pyqtSlot()
    @pyqtSlot(str)
    def show_import(self, app_name: str = None):
        self.setCurrentWidget(self.integrations_page)
        self.integrations_page.show_import(app_name)

    @pyqtSlot()
    def show_egl_sync(self):
        self.setCurrentWidget(self.integrations_page)
        self.integrations_page.show_egl_sync()

    @pyqtSlot()
    def show_eos_ubisoft(self):
        self.setCurrentWidget(self.integrations_page)
        self.integrations_page.show_eos_ubisoft()

    @pyqtSlot(RareGame)
    def show_game_info(self, rgame):
        self.setCurrentWidget(self.game_info_page)
        self.game_info_page.update_game(rgame)

    @pyqtSlot()
    def update_count_games_label(self):
        self.head_bar.set_games_count(
            len([game for game in self.rcore.games if game.is_installed]),
            len([game for game in self.rcore.games])
        )

    def setup_game_list(self):
        for rgame in self.rcore.games:
            icon_widget, list_widget = self.add_library_widget(rgame)
            if not icon_widget or not list_widget:
                logger.warning("Excluding %s from the game list", rgame.app_title)
                continue
            self.icon_view.layout().addWidget(icon_widget)
            self.list_view.layout().addWidget(list_widget)
        self.filter_games(self.active_filter)
        self.update_count_games_label()

    def add_library_widget(self, rgame: RareGame):
        try:
            icon_widget, list_widget = self.library_controller.add_game(rgame)
        except Exception as e:
            logger.error("Could not add widget for %s to library: %s", rgame.app_name, e)
            return None, None
        icon_widget.show_info.connect(self.show_game_info)
        list_widget.show_info.connect(self.show_game_info)
        return icon_widget, list_widget

    @pyqtSlot(str)
    @pyqtSlot(str, str)
    def filter_games(self, filter_name="all", search_text: str = ""):
        if not search_text and (t := self.head_bar.search_bar.text()):
            search_text = t

        if filter_name:
            self.active_filter = filter_name
        if not filter_name and (t := self.active_filter):
            filter_name = t

        self.library_controller.filter_list(filter_name, search_text.lower())

    def toggle_view(self):
        self.settings.setValue("icon_view", not self.head_bar.view.isChecked())

        if not self.head_bar.view.isChecked():
            self.view_stack.slideInWidget(self.icon_view_scroll)
        else:
            self.view_stack.slideInWidget(self.list_view_scroll)



from logging import getLogger

from PyQt5.QtCore import Qt, pyqtSignal, pyqtSlot
from requests.exceptions import ConnectionError, HTTPError

from rare.components.dialogs.login import LoginDialog
from rare.shared import RareCore
from rare.ui.components.dialogs.launch_dialog import Ui_LaunchDialog
from rare.widgets.dialogs import BaseDialog
from rare.widgets.elide_label import ElideLabel

logger = getLogger("LaunchDialog")


class LaunchDialog(BaseDialog):
    # lk: the reason we use the `start_app` signal here instead of accepted, is to keep the dialog
    # until the main window has been created, then we call `accept()` to close the dialog
    start_app = pyqtSignal()

    def __init__(self, parent=None):
        super(LaunchDialog, self).__init__(parent=parent)
        self.setWindowFlags(
            Qt.Window
            | Qt.Dialog
            | Qt.CustomizeWindowHint
            | Qt.WindowSystemMenuHint
            | Qt.WindowTitleHint
            | Qt.WindowMinimizeButtonHint
            | Qt.MSWindowsFixedSizeDialogHint
        )

        self.ui = Ui_LaunchDialog()
        self.ui.setupUi(self)

        self.progress_info = ElideLabel(parent=self)
        self.progress_info.setFixedHeight(False)
        self.ui.launch_layout.addWidget(self.progress_info)

        self.rcore = RareCore.instance()
        self.rcore.progress.connect(self.__on_progress)
        self.rcore.completed.connect(self.__on_completed)
        self.core = self.rcore.core()
        self.args = self.rcore.args()

        self.login_dialog = LoginDialog(core=self.core, parent=parent)
        self.login_dialog.rejected.connect(self.reject)
        self.login_dialog.accepted.connect(self.do_launch)

    def login(self):
        can_launch = True
        try:
            if self.args.offline:
                pass
            else:
                # Force an update check and notice in case there are API changes
                # self.core.check_for_updates(force=True)
                # self.core.force_show_update = True
                if self.core.login(force_refresh=True):
                    logger.info("You are logged in")
                    self.login_dialog.close()
                else:
                    raise ValueError("You are not logged in. Opening login window.")
        except ValueError as e:
            logger.info(str(e))
            # Do not set parent, because it won't show a task bar icon
            # Update: Inherit the same parent as LaunchDialog
            can_launch = False
            self.login_dialog.open()
        except (HTTPError, ConnectionError) as e:
            logger.warning(e)
            self.args.offline = True
        finally:
            if can_launch:
                self.do_launch()

    @pyqtSlot()
    def do_launch(self):
        if not self.args.silent:
            self.open()
        self.launch()

    def launch(self):
        self.progress_info.setText(self.tr("Preparing Rare"))
        self.rcore.fetch()

    @pyqtSlot(int, str)
    def __on_progress(self, i: int, m: str):
        self.ui.progress_bar.setValue(i)
        self.progress_info.setText(m)

    def __on_completed(self):
        logger.info("App starting")
        self.start_app.emit()



import configparser
import os
import time
from argparse import Namespace
from itertools import chain
from logging import getLogger
from typing import Dict, Iterator, Callable, Optional, List, Union, Iterable, Tuple, Set

from PyQt5.QtCore import QObject, pyqtSignal, QSettings, pyqtSlot, QThreadPool, QRunnable, QTimer
from legendary.lfs.eos import EOSOverlayApp
from legendary.models.game import Game, SaveGameFile
from requests import HTTPError

from rare.lgndr.core import LegendaryCore
from rare.models.base_game import RareSaveGame
from rare.models.game import RareGame, RareEosOverlay
from rare.models.signals import GlobalSignals
from .image_manager import ImageManager
from .workers import (
    QueueWorker,
    VerifyWorker,
    MoveWorker,
    FetchWorker,
    OriginWineWorker,
)
from .workers.uninstall import uninstall_game
from .workers.worker import QueueWorkerInfo, QueueWorkerState
from rare.utils import config_helper

logger = getLogger("RareCore")


class RareCore(QObject):
    progress = pyqtSignal(int, str)
    completed = pyqtSignal()
    # lk: these are unused but remain if case they are become relevant
    # completed_saves = pyqtSignal()
    # completed_origin = pyqtSignal()
    # completed_entitlements = pyqtSignal()

    # lk: special case class attribute, this has to be here
    __instance: Optional['RareCore'] = None

    def __init__(self, args: Namespace):
        if self.__instance is not None:
            raise RuntimeError("RareCore already initialized")
        super(RareCore, self).__init__()
        self.__args: Optional[Namespace] = None
        self.__signals: Optional[GlobalSignals] = None
        self.__core: Optional[LegendaryCore] = None
        self.__image_manager: Optional[ImageManager] = None

        self.__start_time = time.time()

        self.args(args)
        self.signals(init=True)
        self.core(init=True)
        config_helper.init_config_handler(self.__core)
        self.image_manager(init=True)

        self.settings = QSettings(self)

        self.queue_workers: List[QueueWorker] = []
        self.queue_threadpool = QThreadPool()
        self.queue_threadpool.setMaxThreadCount(2)

        self.__library: Dict[str, RareGame] = {}
        self.__eos_overlay = RareEosOverlay(self.__core, EOSOverlayApp)

        RareCore.__instance = self

    def enqueue_worker(self, rgame: RareGame, worker: QueueWorker):
        if isinstance(worker, VerifyWorker):
            rgame.state = RareGame.State.VERIFYING
        if isinstance(worker, MoveWorker):
            rgame.state = RareGame.State.MOVING
        rgame.set_worker(worker)
        worker.feedback.started.connect(self.__signals.application.update_statusbar)
        worker.feedback.finished.connect(lambda: rgame.set_worker(None))
        worker.feedback.finished.connect(lambda: self.queue_workers.remove(worker))
        worker.feedback.finished.connect(self.__signals.application.update_statusbar)
        self.queue_workers.append(worker)
        self.queue_threadpool.start(worker, priority=0)
        self.__signals.application.update_statusbar.emit()

    def dequeue_worker(self, worker: QueueWorker):
        rgame = self.__library[worker.worker_info().app_name]
        rgame.set_worker(None)
        self.queue_workers.remove(worker)
        self.__signals.application.update_statusbar.emit()

    def active_workers(self) -> Iterable[QueueWorker]:
        return list(filter(lambda w: w.state == QueueWorkerState.ACTIVE, self.queue_workers))

    def queued_workers(self) -> Iterable[QueueWorker]:
        return list(filter(lambda w: w.state == QueueWorkerState.QUEUED, self.queue_workers))

    def queue_info(self) -> List[QueueWorkerInfo]:
        return [w.worker_info() for w in self.queue_workers]

    @staticmethod
    def instance() -> 'RareCore':
        if RareCore.__instance is None:
            raise RuntimeError("Uninitialized use of RareCore")
        return RareCore.__instance

    def signals(self, init: bool = False) -> GlobalSignals:
        if self.__signals is None and not init:
            raise RuntimeError("Uninitialized use of GlobalSignalsSingleton")
        if self.__signals is not None and init:
            raise RuntimeError("GlobalSignals already initialized")
        if init:
            self.__signals = GlobalSignals()
        return self.__signals

    def args(self, args: Namespace = None) -> Optional[Namespace]:
        if self.__args is None and args is None:
            raise RuntimeError("Uninitialized use of ArgumentsSingleton")
        if self.__args is not None and args is not None:
            raise RuntimeError("Arguments already initialized")
        if args is not None:
            self.__args = args
        return self.__args

    def core(self, init: bool = False) -> LegendaryCore:
        if self.__core is None and not init:
            raise RuntimeError("Uninitialized use of LegendaryCoreSingleton")
        if self.__core is not None and init:
            raise RuntimeError("LegendaryCore already initialized")
        if init:
            try:
                self.__core = LegendaryCore()
            except configparser.MissingSectionHeaderError as e:
                logger.warning("Config is corrupt: %s", e)
                if config_path := os.environ.get('LEGENDARY_CONFIG_PATH'):
                    path = config_path
                elif config_path := os.environ.get('XDG_CONFIG_HOME'):
                    path = os.path.join(config_path, 'legendary')
                else:
                    path = os.path.expanduser('~/.config/legendary')
                logger.info("Creating config in path: %s", config_path)
                with open(os.path.join(path, "config.ini"), "w") as config_file:
                    config_file.write("[Legendary]")
                self.__core = LegendaryCore()

            # Initialize sections if they don't exist
            for section in ["Legendary", "default", "default.env"]:
                if section not in self.__core.lgd.config.sections():
                    self.__core.lgd.config.add_section(section)

            # Set some platform defaults if unset
            def check_config(option: str, accepted: Set = None) -> bool:
                _exists = self.__core.lgd.config.has_option("Legendary", option)
                _value = self.__core.lgd.config.get("Legendary", option, fallback="")
                _accepted = _value in accepted if accepted is not None else True
                return _exists and bool(_value) and _accepted

            if not check_config("default_platform", {"Windows", "Win32", "Mac"}):
                self.__core.lgd.config.set("Legendary", "default_platform", self.__core.default_platform)
            if not check_config("install_dir"):
                self.__core.lgd.config.set(
                    "Legendary", "install_dir", self.__core.get_default_install_dir()
                )
            if not check_config("mac_install_dir"):
                self.__core.lgd.config.set(
                    "Legendary", "mac_install_dir", self.__core.get_default_install_dir(self.__core.default_platform)
                )

            # Always set these options
            # Avoid implicitly falling back to Windows games on macOS
            self.__core.lgd.config.set("Legendary", "install_platform_fallback", str(False))
            # Force-disable automatic use of crossover on macOS (remove this when we support crossover)
            self.__core.lgd.config.set("Legendary", "disable_auto_crossover", str(True))
            # Force-disable automatic sync with EGL, it seems to have issues
            self.__core.lgd.config.set("Legendary", "egl_sync", str(False))

            # workaround if egl sync enabled, but no programdata_path
            # programdata_path might be unset if logging in through the browser
            if self.__core.egl_sync_enabled:
                if self.__core.egl.programdata_path is None:
                    self.__core.lgd.config.remove_option("Legendary", "egl_sync")
                else:
                    if not os.path.exists(self.__core.egl.programdata_path):
                        self.__core.lgd.config.remove_option("Legendary", "egl_sync")

            self.__core.lgd.save_config()
        return self.__core

    def image_manager(self, init: bool = False) -> ImageManager:
        if self.__image_manager is None and not init:
            raise RuntimeError("Uninitialized use of ImageManagerSingleton")
        if self.__image_manager is not None and init:
            raise RuntimeError("ImageManager already initialized")
        if self.__image_manager is None:
            self.__image_manager = ImageManager(self.signals(), self.core())
        return self.__image_manager

    def deleteLater(self) -> None:
        self.__image_manager.deleteLater()
        del self.__image_manager
        self.__image_manager = None

        self.__core.exit()
        del self.__core
        self.__core = None

        self.__signals.deleteLater()
        del self.__signals
        self.__signals = None

        del self.__args
        self.__args = None

        # del self.__eos_overlay
        self.__eos_overlay = None

        RareCore.__instance = None
        super(RareCore, self).deleteLater()

    def __validate_install(self, rgame: RareGame):
        if not os.path.exists(rgame.igame.install_path):
            # lk: since install_path is lost anyway, set keep_files to True
            # lk: to avoid spamming the log with "file not found" errors
            for dlc in rgame.owned_dlcs:
                if dlc.is_installed:
                    logger.info(f'Uninstalling DLC "{dlc.app_name}" ({dlc.app_title})...')
                    uninstall_game(self.__core, dlc.app_name, keep_files=True, keep_config=True)
                    dlc.igame = None
            logger.info(
                f'Removing "{rgame.app_title}" because "{rgame.igame.install_path}" does not exist...'
            )
            uninstall_game(self.__core, rgame.app_name, keep_files=True, keep_config=True)
            logger.info(f"Uninstalled {rgame.app_title}, because no game files exist")
            rgame.igame = None
            return
        # lk: games that don't have an override and can't find their executable due to case sensitivity
        # lk: will still erroneously require verification. This might need to be removed completely
        # lk: or be decoupled from the verification requirement
        if override_exe := self.__core.lgd.config.get(rgame.app_name, "override_exe", fallback=""):
            igame_executable = override_exe
        else:
            igame_executable = rgame.igame.executable
        # lk: Case-insensitive search for the game's executable (example: Brothers - A Tale of two Sons)
        executable_path = os.path.join(rgame.igame.install_path, igame_executable.replace("\\", "/").lstrip("/"))
        file_list = map(str.lower, os.listdir(os.path.dirname(executable_path)))
        if not os.path.basename(executable_path).lower() in file_list:
            rgame.igame.needs_verification = True
            self.__core.lgd.set_installed_game(rgame.app_name, rgame.igame)
            rgame.update_igame()
            logger.info(f"{rgame.app_title} needs verification")

    def get_game(self, app_name: str) -> Union[RareEosOverlay, RareGame]:
        if app_name == EOSOverlayApp.app_name:
            return self.__eos_overlay
        return self.__library[app_name]

    def __add_game(self, rgame: RareGame) -> None:
        rgame.signals.download.enqueue.connect(self.__signals.download.enqueue)
        rgame.signals.download.dequeue.connect(self.__signals.download.dequeue)
        rgame.signals.game.install.connect(self.__signals.game.install)
        rgame.signals.game.installed.connect(self.__signals.game.installed)
        rgame.signals.game.uninstall.connect(self.__signals.game.uninstall)
        rgame.signals.game.uninstalled.connect(self.__signals.game.uninstalled)
        rgame.signals.game.finished.connect(self.__signals.application.update_tray)
        rgame.signals.game.finished.connect(lambda: self.__signals.discord_rpc.set_title.emit(""))
        self.__library[rgame.app_name] = rgame

    def __filter_games(self, condition: Callable[[RareGame], bool]) -> Iterator[RareGame]:
        return filter(condition, self.__library.values())

    def __create_or_update_rgame(self, game: Game) -> RareGame:
        if rgame := self.__library.get(game.app_name, False):
            logger.warning(f"{rgame.app_name} already present in {type(self).__name__}")
            logger.info(f"Updating Game for {rgame.app_name}")
            rgame.update_rgame()
        else:
            rgame = RareGame(self.__core, self.__image_manager, game)
            self.__add_game(rgame)
        return rgame

    def __add_games_and_dlcs(self, games: List[Game], dlcs_dict: Dict[str, List]) -> None:
        length = len(games)
        for idx, game in enumerate(games):
            rgame = self.__create_or_update_rgame(game)
            if game_dlcs := dlcs_dict.get(rgame.game.catalog_item_id, False):
                for dlc in game_dlcs:
                    rdlc = self.__create_or_update_rgame(dlc)
                    if rdlc not in rgame.owned_dlcs:
                        rgame.add_dlc(rdlc)
            # lk: since loading has to know about game state,
            # validate installation just adding each RareGamesu
            # TODO: this should probably be moved into RareGame
            if rgame.is_installed and not (rgame.is_dlc or rgame.is_non_asset):
                try:
                    self.__validate_install(rgame)
                except FileNotFoundError as e:
                    logger.info(f'Marking "{rgame.app_title}" as not installed because an exception has occurred...')
                    logger.error(e)
                    rgame.set_installed(False)
            self.progress.emit(int(idx/length * 80) + 20, self.tr("Loaded <b>{}</b>").format(rgame.app_title))

    @pyqtSlot(object, int)
    def __on_fetch_result(self, result: Tuple[List, Dict], res_type: int):
        logger.info(f"Got API results for {FetchWorker.Result(res_type).name}")
        self.progress.emit(15, self.tr("Preparing library"))
        self.__add_games_and_dlcs(*result)
        self.progress.emit(100, self.tr("Launching Rare"))
        logger.debug(f"Fetch time {time.time() - self.__start_time} seconds")
        QTimer.singleShot(100, self.__post_init)
        self.completed.emit()

    def fetch(self):
        self.__start_time = time.time()
        fetch_worker = FetchWorker(self.__core, self.__args)
        fetch_worker.signals.progress.connect(self.progress)
        fetch_worker.signals.result.connect(self.__on_fetch_result)
        QThreadPool.globalInstance().start(fetch_worker)

    def fetch_saves(self):
        def __fetch() -> None:
            start_time = time.time()
            saves_dict: Dict[str, List[SaveGameFile]] = {}
            try:
                saves_list = self.__core.get_save_games()
                for s in saves_list:
                    if s.app_name not in saves_dict.keys():
                        saves_dict[s.app_name] = [s]
                    else:
                        saves_dict[s.app_name].append(s)
                for app_name, saves in saves_dict.items():
                    if app_name not in self.__library.keys():
                        continue
                    self.__library[app_name].load_saves(saves)
            except (HTTPError, ConnectionError) as e:
                logger.error(f"Exception while fetching saves from EGS: {e}")
                return
            logger.debug(f"Saves: {len(saves_dict)}")
            logger.debug(f"Request saves: {time.time() - start_time} seconds")

        saves_worker = QRunnable.create(__fetch)
        QThreadPool.globalInstance().start(saves_worker)

    def fetch_entitlements(self) -> None:
        def __fetch() -> None:
            start_time = time.time()
            try:
                entitlements = self.__core.egs.get_user_entitlements()
                self.__core.lgd.entitlements = entitlements
                for game in self.__library.values():
                    game.grant_date()
            except (HTTPError, ConnectionError) as e:
                logger.error(f"Failed to retrieve user entitlements from EGS: {e}")
                return
            logger.debug(f"Entitlements: {len(list(entitlements))}")
            logger.debug(f"Request Entitlements: {time.time() - start_time} seconds")

        entitlements_worker = QRunnable.create(__fetch)
        QThreadPool.globalInstance().start(entitlements_worker)

    def resolve_origin(self) -> None:
        origin_worker = OriginWineWorker(self.__core, list(self.origin_games))
        QThreadPool.globalInstance().start(origin_worker)

    def __post_init(self) -> None:
        if not self.__args.offline:
            self.fetch_saves()
            self.fetch_entitlements()
        self.resolve_origin()

    @property
    def games_and_dlcs(self) -> Iterator[RareGame]:
        for app_name in self.__library:
            yield self.__library[app_name]

    @property
    def games(self) -> Iterator[RareGame]:
        return self.__filter_games(lambda game: not game.is_dlc)

    @property
    def installed_games(self) -> Iterator[RareGame]:
        return self.__filter_games(lambda game: game.is_installed and not game.is_dlc)

    @property
    def origin_games(self) -> Iterator[RareGame]:
        return self.__filter_games(lambda game: game.is_origin and not game.is_dlc)

    @property
    def ubisoft_games(self) -> Iterator[RareGame]:
        return self.__filter_games(lambda game: game.is_ubisoft and not game.is_dlc)

    @property
    def game_list(self) -> Iterator[Game]:
        for game in self.games:
            yield game.game

    @property
    def dlcs(self) -> Dict[str, set[RareGame]]:
        """!
        RareGames that ARE DLCs themselves
        """
        return {game.game.catalog_item_id: game.owned_dlcs for game in self.has_dlcs}
        # return self.__filter_games(lambda game: game.is_dlc)

    @property
    def has_dlcs(self) -> Iterator[RareGame]:
        """!
        RareGames that HAVE DLCs associated with them
        """
        return self.__filter_games(lambda game: bool(game.owned_dlcs))

    @property
    def bit32_games(self) -> Iterator[RareGame]:
        return self.__filter_games(lambda game: game.is_win32)

    @property
    def mac_games(self) -> Iterator[RareGame]:
        return self.__filter_games(lambda game: game.is_mac)

    @property
    def non_asset_games(self) -> Iterator[RareGame]:
        return self.__filter_games(lambda game: game.is_non_asset)

    @property
    def unreal_engine(self) -> Iterator[RareGame]:
        return self.__filter_games(lambda game: game.is_unreal)

    @property
    def updates(self) -> Iterator[RareGame]:
        return self.__filter_games(lambda game: game.has_update)

    @property
    def saves(self) -> Iterator[RareSaveGame]:
        """!
        SaveGameFiles across games
        """
        return chain.from_iterable([game.saves for game in self.has_saves])

    @property
    def has_saves(self) -> Iterator[RareGame]:
        """!
        RareGames that have SaveGameFiles associated with them
        """
        return self.__filter_games(lambda game: bool(game.saves))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/dialogs/login/browser_login.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_BrowserLogin(object):
    def setupUi(self, BrowserLogin):
        BrowserLogin.setObjectName("BrowserLogin")
        BrowserLogin.resize(320, 243)
        BrowserLogin.setWindowTitle("BrowserLogin")
        self.main_layout = QtWidgets.QVBoxLayout(BrowserLogin)
        self.main_layout.setObjectName("main_layout")
        self.title_label = QtWidgets.QLabel(BrowserLogin)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.title_label.sizePolicy().hasHeightForWidth())
        self.title_label.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.title_label.setFont(font)
        self.title_label.setObjectName("title_label")
        self.main_layout.addWidget(self.title_label)
        self.form_layout = QtWidgets.QFormLayout()
        self.form_layout.setSizeConstraint(QtWidgets.QLayout.SetFixedSize)
        self.form_layout.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.form_layout.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.form_layout.setFormAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.form_layout.setObjectName("form_layout")
        self.open_button = QtWidgets.QPushButton(BrowserLogin)
        self.open_button.setObjectName("open_button")
        self.form_layout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.open_button)
        self.link_layout = QtWidgets.QHBoxLayout()
        self.link_layout.setObjectName("link_layout")
        self.link_text = QtWidgets.QLineEdit(BrowserLogin)
        self.link_text.setText("")
        self.link_text.setReadOnly(True)
        self.link_text.setPlaceholderText("")
        self.link_text.setObjectName("link_text")
        self.link_layout.addWidget(self.link_text)
        self.copy_button = QtWidgets.QPushButton(BrowserLogin)
        self.copy_button.setText("")
        self.copy_button.setObjectName("copy_button")
        self.link_layout.addWidget(self.copy_button)
        self.link_layout.setStretch(0, 1)
        self.form_layout.setLayout(0, QtWidgets.QFormLayout.FieldRole, self.link_layout)
        self.sid_label = QtWidgets.QLabel(BrowserLogin)
        self.sid_label.setText("authorizationCode")
        self.sid_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sid_label.setObjectName("sid_label")
        self.form_layout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.sid_label)
        self.status_label = QtWidgets.QLabel(BrowserLogin)
        font = QtGui.QFont()
        font.setItalic(True)
        self.status_label.setFont(font)
        self.status_label.setText("")
        self.status_label.setObjectName("status_label")
        self.form_layout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.status_label)
        self.main_layout.addLayout(self.form_layout)
        spacerItem = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.main_layout.addItem(spacerItem)
        self.info_label = QtWidgets.QLabel(BrowserLogin)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.info_label.sizePolicy().hasHeightForWidth())
        self.info_label.setSizePolicy(sizePolicy)
        self.info_label.setWordWrap(True)
        self.info_label.setObjectName("info_label")
        self.main_layout.addWidget(self.info_label)

        self.retranslateUi(BrowserLogin)

    def retranslateUi(self, BrowserLogin):
        _translate = QtCore.QCoreApplication.translate
        self.title_label.setText(_translate("BrowserLogin", "Login through browser"))
        self.open_button.setText(_translate("BrowserLogin", "Open Browser"))
        self.info_label.setText(_translate("BrowserLogin", "<i>Click the <strong>Open Browser</strong> button to open the login page in your web browser or copy the link and paste it in any web browser. After logging in using the browser, copy the text in the quotes after </i><code><b>authorizationCode</b></code><i> in the same line into the empty input above.<br><br><strong>DO NOT SHARE THE INFORMATION IN THE BROWSER PAGE WITH ANYONE IN ANY FORM (TEXT OR SCREENSHOT)!</strong></i>"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    BrowserLogin = QtWidgets.QWidget()
    ui = Ui_BrowserLogin()
    ui.setupUi(BrowserLogin)
    BrowserLogin.show()
    sys.exit(app.exec_())



import time
import webbrowser
from logging import getLogger
from typing import Optional

from PyQt5.QtCore import QObject, pyqtSignal, QThreadPool, QSize, pyqtSlot, Qt
from PyQt5.QtGui import QShowEvent
from PyQt5.QtWidgets import (
    QFrame,
    QLabel,
    QHBoxLayout,
    QSizePolicy,
    QPushButton,
    QGroupBox,
    QVBoxLayout,
)
from legendary.models.game import Game

from rare.lgndr.core import LegendaryCore
from rare.shared import RareCore
from rare.shared.workers.worker import Worker
from rare.utils.metrics import timelogger
from rare.utils.misc import icon
from rare.widgets.elide_label import ElideLabel
from rare.widgets.loading_widget import LoadingWidget

logger = getLogger("Ubisoft")


class UbiGetInfoWorker(Worker):
    class Signals(QObject):
        worker_finished = pyqtSignal(set, set, str)

    def __init__(self, core: LegendaryCore):
        super(UbiGetInfoWorker, self).__init__()
        self.signals = UbiGetInfoWorker.Signals()
        self.setAutoDelete(True)
        self.core = core

    def run_real(self) -> None:
        try:
            with timelogger(logger, "Request external auths"):
                external_auths = self.core.egs.get_external_auths()
            for ext_auth in external_auths:
                if ext_auth["type"] != "ubisoft":
                    continue
                ubi_account_id = ext_auth["externalAuthId"]
                break
            else:
                self.signals.worker_finished.emit(set(), set(), "")
                return

            with timelogger(logger, "Request uplay codes"):
                uplay_keys = self.core.egs.store_get_uplay_codes()
            key_list = uplay_keys["data"]["PartnerIntegration"]["accountUplayCodes"]
            redeemed = {k["gameId"] for k in key_list if k["redeemedOnUplay"]}

            if (entitlements := self.core.lgd.entitlements) is None:
                with timelogger(logger, "Request entitlements"):
                    entitlements = self.core.egs.get_user_entitlements()
                self.core.lgd.entitlements = entitlements
            entitlements = {i["entitlementName"] for i in entitlements}

            self.signals.worker_finished.emit(redeemed, entitlements, ubi_account_id)
        except Exception as e:
            logger.error(str(e))
            self.signals.worker_finished.emit(set(), set(), "error")


class UbiConnectWorker(Worker):
    class Signals(QObject):
        linked = pyqtSignal(str)

    def __init__(self, core: LegendaryCore, ubi_account_id, partner_link_id):
        super(UbiConnectWorker, self).__init__()
        self.signals = UbiConnectWorker.Signals()
        self.core = core
        self.ubi_account_id = ubi_account_id
        self.partner_link_id = partner_link_id

    def run_real(self) -> None:
        if not self.ubi_account_id:  # debug
            time.sleep(2)
            self.signals.linked.emit("")
            return
        try:
            self.core.egs.store_claim_uplay_code(self.ubi_account_id, self.partner_link_id)
            self.core.egs.store_redeem_uplay_codes(self.ubi_account_id)
        except Exception as e:
            self.signals.linked.emit(str(e))
            return
        else:
            self.signals.linked.emit("")


class UbiLinkWidget(QFrame):
    def __init__(self, game: Game, ubi_account_id, activated: bool = False, parent=None):
        super(UbiLinkWidget, self).__init__(parent=parent)
        self.setFrameShape(QFrame.StyledPanel)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)

        self.args = RareCore.instance().args()
        self.game = game
        self.ubi_account_id = ubi_account_id

        self.ok_indicator = QLabel(parent=self)
        self.ok_indicator.setPixmap(icon("fa.circle-o", color="grey").pixmap(20, 20))
        self.ok_indicator.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Preferred)

        self.title_label = ElideLabel(game.app_title, parent=self)

        self.link_button = QPushButton(self.tr("Redeem in Ubisoft"), parent=self)
        self.link_button.setMinimumWidth(150)
        self.link_button.clicked.connect(self.activate)

        if activated:
            self.link_button.setText(self.tr("Already activated"))
            self.link_button.setDisabled(True)
            self.ok_indicator.setPixmap(icon("fa.check-circle-o", color="green").pixmap(QSize(20, 20)))

        layout = QHBoxLayout(self)
        layout.setContentsMargins(-1, 0, 0, 0)
        layout.addWidget(self.ok_indicator)
        layout.addWidget(self.title_label, stretch=1)
        layout.addWidget(self.link_button)

    def activate(self):
        self.link_button.setDisabled(True)
        # self.ok_indicator.setPixmap(icon("mdi.loading", color="grey").pixmap(20, 20))
        self.ok_indicator.setPixmap(icon("mdi.transit-connection-horizontal", color="grey").pixmap(20, 20))

        if self.args.debug:
            worker = UbiConnectWorker(RareCore.instance().core(), None, None)
        else:
            worker = UbiConnectWorker(
                RareCore.instance().core(), self.ubi_account_id, self.game.partner_link_id
            )
        worker.signals.linked.connect(self.worker_finished)
        QThreadPool.globalInstance().start(worker)

    def worker_finished(self, error):
        if not error:
            self.ok_indicator.setPixmap(icon("fa.check-circle-o", color="green").pixmap(QSize(20, 20)))
            self.link_button.setDisabled(True)
            self.link_button.setText(self.tr("Already activated"))
        else:
            self.ok_indicator.setPixmap(icon("fa.times-circle-o", color="red").pixmap(QSize(20, 20)))
            self.ok_indicator.setToolTip(error)
            self.link_button.setText(self.tr("Try again"))
            self.link_button.setDisabled(False)


class UbisoftGroup(QGroupBox):
    def __init__(self, parent=None):
        super(UbisoftGroup, self).__init__(parent=parent)
        self.setTitle(self.tr("Link Ubisoft Games"))
        self.rcore = RareCore.instance()
        self.core = RareCore.instance().core()
        self.args = RareCore.instance().args()

        self.thread_pool = QThreadPool.globalInstance()
        self.worker: Optional[UbiGetInfoWorker] = None

        self.info_label = QLabel(parent=self)
        self.info_label.setText(self.tr("Getting information about your redeemable Ubisoft games."))
        self.browser_button = QPushButton(self.tr("Link Ubisoft acccount"), parent=self)
        self.browser_button.setMinimumWidth(140)
        self.browser_button.clicked.connect(
            lambda: webbrowser.open("https://www.epicgames.com/id/link/ubisoft")
        )
        self.browser_button.setEnabled(False)

        self.loading_widget = LoadingWidget(self)
        self.loading_widget.stop()

        header_layout = QHBoxLayout()
        header_layout.addWidget(self.info_label, stretch=1)
        header_layout.addWidget(self.browser_button)

        layout = QVBoxLayout(self)
        layout.addLayout(header_layout)
        layout.addWidget(self.loading_widget)

    def showEvent(self, a0: QShowEvent) -> None:
        if a0.spontaneous():
            return super().showEvent(a0)

        if self.worker is not None:
            return

        for widget in self.findChildren(UbiLinkWidget, options=Qt.FindDirectChildrenOnly):
            widget.deleteLater()
        self.loading_widget.start()

        self.worker = UbiGetInfoWorker(self.core)
        self.worker.signals.worker_finished.connect(self.show_ubi_games)
        self.thread_pool.start(self.worker)
        super().showEvent(a0)

    @pyqtSlot(set, set, str)
    def show_ubi_games(self, redeemed: set, entitlements: set, ubi_account_id: str):
        self.worker = None
        if not redeemed and ubi_account_id != "error":
            logger.error(
                "No linked ubisoft account found! Link your accounts via your browser and try again."
            )
            self.info_label.setText(
                self.tr("Your account is not linked with Ubisoft. Please link your account and try again.")
            )
            self.browser_button.setEnabled(True)
        elif ubi_account_id == "error":
            self.info_label.setText(
                self.tr("An error has occurred while requesting your account's Ubisoft information.")
            )
            self.browser_button.setEnabled(True)
        else:
            self.browser_button.setEnabled(False)

            uplay_games = []
            activated = 0
            for rgame in self.rcore.ubisoft_games:
                game = rgame.game
                for dlc_data in game.metadata.get("dlcItemList", []):
                    if dlc_data["entitlementName"] not in entitlements:
                        continue
                    try:
                        app_name = dlc_data["releaseInfo"][0]["appId"]
                    except (IndexError, KeyError):
                        app_name = "unknown"

                    dlc_game = Game(app_name=app_name, app_title=dlc_data["title"], metadata=dlc_data)
                    if dlc_game.partner_link_type != "ubisoft":
                        continue
                    if dlc_game.partner_link_id in redeemed:
                        continue
                    uplay_games.append(dlc_game)

                if game.partner_link_type != "ubisoft":
                    continue
                if game.partner_link_id in redeemed:
                    activated += 1
                uplay_games.append(game)

            if not uplay_games:
                self.info_label.setText(self.tr("You don't own any Ubisoft games."))
            else:
                if activated == len(uplay_games):
                    self.info_label.setText(self.tr("All your Ubisoft games have already been activated."))
                else:
                    self.info_label.setText(
                        self.tr("You have <b>{}</b> games available to redeem.").format(
                            len(uplay_games) - activated
                        )
                    )
            logger.info(f"Found {len(uplay_games) - activated} game(s) to redeem.")

            self.loading_widget.stop()

            for game in uplay_games:
                widget = UbiLinkWidget(
                    game, ubi_account_id, activated=game.partner_link_id in redeemed, parent=self
                )
                self.layout().addWidget(widget)

            if self.args.debug:
                widget = UbiLinkWidget(
                    Game(app_name="RareTestGame", app_title="Super Fake Super Rare Super Test (Super?) Game"),
                    ubi_account_id,
                    activated=False,
                    parent=self,
                )
                self.layout().addWidget(widget)
                self.browser_button.setEnabled(True)
